/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.dtypes;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import at.lcm.bu21.general.utils.Clone;
import at.lcm.bu21.general.utils.CloneException;

/**
 * Representation of a boolean-type
 * 
 * @author Werner Koppelsttter
 */
public class TBoolean extends TData {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Data variable */
	protected boolean data = false;

	/**
	 * Create empty object to loading
	 */
	protected TBoolean() {
	}

	/**
	 * Create object with given data
	 * 
	 * @param bool
	 *            Data for object
	 */
	public TBoolean(boolean bool) {
		this.data = bool;
	}

	/**
	 * Create object from string
	 * 
	 * @param bool
	 *            Boolean string
	 * @throws TDataException
	 *             Exceptino by string convertion
	 */
	public TBoolean(String bool) throws TDataException {
		bool = bool.trim();
		if (bool.compareToIgnoreCase("true") == 0) {
			this.data = true;
		} else if (bool.compareToIgnoreCase("false") == 0) {
			this.data = false;
		} else {
			throw new TDataException("String is no boolean");
		}
	}

	@Override
	public TBoolean deepClone() throws CloneException {
		return (TBoolean) Clone.deepClone(this);
	}

	/**
	 * Get the data content
	 * 
	 * @return Data of object
	 */
	public boolean getBoolean() {
		return this.data;
	}

	@Override
	public void xml2data(XMLStreamReader xmlReader) throws XMLStreamException {

		xmlReader.next();
		this.data = Boolean.parseBoolean(xmlReader.getText());
		xmlReader.next();
		xmlReader.next();
	}

	@Override
	public void data2xml(XMLStreamWriter xmlWriter) throws XMLStreamException {
		xmlWriter.writeStartElement("Boolean");
		xmlWriter.writeCharacters(Boolean.toString(this.data));
		xmlWriter.writeEndElement();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (this.getClass() != obj.getClass()) {
			return false;
		}
		TBoolean other = (TBoolean) obj;
		if (this.data != other.data) {
			return false;
		}
		return true;
	}

	/**
	 * Create a string representation
	 * 
	 * @return String representation
	 */
	@Override
	public String toString() {
		return Boolean.toString(this.data);
	}

}
