/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor;

import java.io.File;

import javax.swing.JPanel;

import at.lcm.bu21.general.dtypes.TData;

/**
 * EditorPanels Interface
 * 
 * @author Christoph Halbmayr
 */
interface EditorPanelInterface {

	/**
	 * deletes the selected TData/Signal
	 */
	public void delete();

	/**
	 * deletes the selected column
	 */
	public void deleteColumn();

	/**
	 * deletes the selected row
	 */
	public void deleteRow();

	/**
	 * returns the JPanel
	 * 
	 * @return JPanel
	 */
	public JPanel getPanel();

	/**
	 * returns the selected TData
	 * 
	 * @return TData
	 */
	public TData getSelectedTData();

	/**
	 * returns the TData
	 * 
	 * @return TData
	 */
	public TData getTData();

	/**
	 * import TData from file
	 * 
	 * @param f
	 *            source File
	 */
	public void importTData(File f);

	/**
	 * inserts a Column
	 */
	public void insertColumn();

	/**
	 * inserts a Row
	 */
	public void insertRow();

	/**
	 * prints out the whole table
	 */
	public void printme();

	/**
	 * reloads/repaints the whole panel/table
	 */
	public void reload();

	/**
	 * exports TData to file
	 * 
	 * @param f
	 *            destiantion File
	 */
	public void saveTData(File f);

	/**
	 * updates the selectet TData
	 * 
	 * @param selection
	 *            TData
	 */
	public void updateChangedSignal(TData selection);

}
