/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor;

import java.util.Vector;

/**
 * EBNF Parser
 * 
 * @author Christoph Halbmayr
 */
public class TPeriodicGeneratorParser {

	/**
	 * Removes the Brackets from '(String)' to 'String'
	 * 
	 * @param input
	 *            String in Brackets
	 * @return String whitout Brackets
	 */
	public static String removeBrackets(String input) {

		input = input.replace(" ", "");

		int counts = 0;
		while ((TPeriodicGeneratorParser.splitProduct(input).size() != 3)
				&& (TPeriodicGeneratorParser.splitSum(input).size() != 3) && (counts < 50)) {
			if (input.startsWith("(") && input.endsWith(")")) {
				input = input.substring(1, input.length() - 1);
			}
			counts++;
		}
		return input;

	}

	/**
	 * Splits a Product into 3 Parts A*B (0)=A (1)=* (2)=B
	 * 
	 * @param input
	 *            String
	 * @return splitted parts
	 */
	public static Vector<String> splitProduct(final String input) {

		// System.out.println("prod IN: " + intput);

		final Vector<String> split = new Vector<String>();

		String segment = new String();

		for (int i = 0; i < input.length(); i++) {

			final char in = input.toCharArray()[i];

			if (((in != '*') && (in != '/') && (i < (input.length() - 1)))
					|| ((split.size() >= 2) && (i < (input.length() - 1)))) {
				segment = segment + in;
			} else if (!segment.contains("(") && !segment.contains(")")) {
				if (i == (input.length() - 1)) {
					segment = segment + in;
				}
				split.add(segment);
				if ((in == '*') || (in == '/')) {
					split.add("" + in);
				}
				segment = "";
			} else if (input.toCharArray()[i - 1] == ')') {
				split.add(segment);
				segment = "";
				split.add("" + in);
			} else if (i == (input.length() - 1)) {
				segment = segment + in;
				split.add(segment);
				segment = "";
			} else {
				segment = segment + in;
			}

		}

		// System.out.println("prod OUT: " + split);

		if (split.size() == 0) {
			return null;
		} else {
			return split;
		}
	}

	/**
	 * Splits a Sum into 3 Parts A+B (0)=A (1)=+ (2)=B
	 * 
	 * @param input
	 *            String
	 * @return splitted parts
	 */
	public static Vector<String> splitSum(final String input) {

		// System.out.println("sum IN: " + intput);

		final Vector<String> split = new Vector<String>();

		String segment = new String();

		for (int i = 0; i < input.length(); i++) {

			final char in = input.toCharArray()[i];

			if (((in != '+') && (in != '-') && (i < (input.length() - 1)))
					|| ((split.size() >= 2) && (i < (input.length() - 1)))) {
				segment = segment + in;
			} else if ((segment.contains("(") == false) && (segment.contains(")") == false)) {

				if (i == (input.length() - 1)) {
					segment = segment + in;
				}
				split.add(segment);
				if ((in == '+') || (in == '-')) {
					split.add("" + in);
				}

				segment = "";
			} else if (input.toCharArray()[i - 1] == ')') {

				split.add(segment);
				segment = "";

				split.add("" + in);

			} else if (i == (input.length() - 1)) {
				segment = segment + in;
				split.add(segment);
				segment = "";
			} else {
				segment = segment + in;
			}

		}

		// System.out.println("sum OUT: " + split);

		if (split.size() == 0) {
			return null;
		} else {
			return split;
		}
	}
}
