/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor;

import java.util.Vector;

import javax.swing.table.DefaultTableModel;

import at.lcm.bu21.general.dtypes.TArray;
import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TDataException;
import at.lcm.bu21.general.dtypes.TPeriodicMatrix;

/**
 * @author Christoph Halbmayr
 */
class TableModelMatrixPanel extends DefaultTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Creats an empty Object
	 */
	TableModelMatrixPanel() {
		super();
	}

	/**
	 * Creats an Object
	 * 
	 * @param columnNames
	 *            titels
	 * @param rowCount
	 *            row Count
	 */
	TableModelMatrixPanel(final Vector<?> columnNames, final int rowCount) {
		super(columnNames, rowCount);

	}

	/**
	 * adds a TData to the TableModel
	 * 
	 * @param data
	 */
	public void addTData(final TData data) {

		if (data.getClass().getName().equals("at.lcm.bu21.general.dtypes.TArray")) {
			this.addTArray((TArray) data);
		} else if (data.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodicMatrix")) {
			this.addTPeriodicMatrix((TPeriodicMatrix) data);
		}

	}

	@Override
	public Object getValueAt(final int row, final int column) {

		return super.getValueAt(row, column);
	}

	@Override
	public boolean isCellEditable(final int row, final int column) {

		return false;
	}

	@Override
	public void setValueAt(final Object aValue, final int rowIndex, final int columnIndex) {

		try {
			super.setValueAt(aValue, rowIndex, columnIndex);
		} catch (final Exception e) {
		}

	}

	/**
	 * adds a TArray to the TableModel
	 * 
	 * @param data
	 *            TArray
	 */
	private void addTArray(final TArray data) {
		for (int m = 0; m < data.size()[0]; m++) {
			final Vector<String> ss = new Vector<String>();
			for (int n = 0; n < data.size()[1]; n++) {
				try {

					ss.add(data.get(m, n).toString());
				} catch (final TDataException e) {
					e.printStackTrace();
				}
			}
			this.addRow(ss);

		}
	}

	/**
	 * adds a TPeriodicMatrix to the TableModel
	 * 
	 * @param data
	 *            TPeriodicMatrix
	 */
	private void addTPeriodicMatrix(final TPeriodicMatrix data) {
		for (int m = 0; m < data.size()[0]; m++) {
			final Vector<String> ss = new Vector<String>();
			for (int n = 0; n < data.size()[1]; n++) {
				try {

					ss.add(data.get(m, n).toString());
				} catch (final TDataException e) {
					e.printStackTrace();
				}
			}
			this.addRow(ss);

		}
	}
}
