/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2;

import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.general.dtypes.editor2.TDataModel.TArrayModel;

/**
 * This is a table model for a JTable to view TArrays. It based on
 * AbstractTableModel.
 * 
 * @author Johannes Kaisinger
 */
public class TArrayTableModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Local reference of the tArrayModel. It allows to manipulate the TArray.
	 */
	private TArrayModel tArrayModel;

	/**
	 * Creates an object
	 * 
	 * @param tArrayModel
	 *            The TArrayModel
	 */
	public TArrayTableModel(TArrayModel tArrayModel) {
		this.tArrayModel = tArrayModel;
	}

	/**
	 * Set tArrayModel
	 * 
	 * @param tArrayModel
	 *            The TArrayModel
	 */
	public void setData(TArrayModel tArrayModel) {
		this.tArrayModel = tArrayModel;
	}

	/**
	 * Creates an object
	 * 
	 * @param column
	 *            Index of column.
	 * @return name The name of the column
	 */
	@Override
	public String getColumnName(int column) {
		return "" + column;
	}

	/**
	 * Get row count of tArrayModel.
	 * 
	 * @return rowCount
	 */
	@Override
	public int getRowCount() {
		int rowCount = this.tArrayModel.getTArray().size()[0];
		return rowCount;
	}

	/**
	 * Get column count of tArrayModel.
	 * 
	 * @return columnCount
	 */
	@Override
	public int getColumnCount() {
		int columnCount = this.tArrayModel.getTArray().size()[1];
		return columnCount;
	}

	/**
	 * Get column class.
	 * 
	 * @param column
	 *            Index of column.
	 * @return Class
	 */
	@Override
	public Class<String> getColumnClass(int column) {
		return String.class;
	}

	/**
	 * Get if a cell is editable.
	 * 
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 * @return isEditable
	 */
	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}

	/**
	 * Get a cell value.
	 * 
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 * @return value Value of cell (String).
	 */
	@Override
	public String getValueAt(int row, int column) {
		String value = this.tArrayModel.getValue(row, column);
		return value;
	}

}
