/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2.TDataModel;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TString;

/**
 * Representation of a TStringModel - type.
 * 
 * @author Johannes Kaisinger
 */
public class TStringModel extends TDataModel {

	/** Data variable */
	private TString data;

	/**
	 * Create object with empty String
	 * 
	 */
	public TStringModel() {
		this.data = new TString("");
	}

	/**
	 * Create object with given data (TString)
	 * 
	 * @param data
	 *            Reference of data (TString)
	 * 
	 */
	public TStringModel(TString data) {
		super();
		this.data = data;

	}

	/**
	 * Get data (TString)
	 * 
	 * @return Data Reference of data (TString)
	 */
	public String getString() {
		return this.data.getString();
	}

	/**
	 * Get data (TData)
	 * 
	 * @return data Reference of data (TData)
	 */
	@Override
	public TData getData() {
		return this.data;
	}

	/**
	 * Set data (String)
	 * 
	 * @param string
	 *            New value of data
	 */
	public void setString(String string) {
		this.data = new TString(string);
	}
}
