/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.plot;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;

/**
 * This class represents a Marker for the Style Class
 * 
 * @author Christoph Halbmayr
 * @author Werner Koppelsttter
 */
public enum Marker {

	/** Circle */
	CIRLE,
	/** Square */
	SQUARE,
	/** Diamond */
	DIAMOND,
	/** Upward-pointing triangle */
	TRIANGLE_UP,
	/** Downward-pointing triangle */
	TRIANGLE_DOWN,
	/** Right-pointing triangle */
	TRIANGLE_RIGHT,
	/** Left-pointing triangle */
	TRIANGLE_LEFT,
	/** Plus sign */
	PLUS,
	/** Cross */
	CROSS;

	/**
	 * Returns the marker-shape
	 * 
	 * @param marker
	 *            Type of marker
	 * @param size
	 *            Size of the marker
	 * @return Marker shape
	 */
	public static Shape getShape(Marker marker, int size) {
		int len = size / 2;
		Shape shape = null;

		switch (marker) {
		case CIRLE:
			shape = new Ellipse2D.Float(-len, -len, 2 * len, 2 * len);
			break;
		case SQUARE:
			shape = new Rectangle(-len, -len, 2 * len, 2 * len);
			break;
		case DIAMOND:
			shape = new Polygon(new int[] { -len, 0, len, 0 }, new int[] { 0,
					len, 0, -len }, 4);
			break;
		case TRIANGLE_UP:
			shape = new Polygon(new int[] { -len, 0, len }, new int[] { -len,
					len, -len }, 3);
			break;
		case TRIANGLE_DOWN:
			shape = new Polygon(new int[] { -len, len, 0 }, new int[] { len,
					len, -len }, 3);
			break;
		case TRIANGLE_RIGHT:
			shape = new Polygon(new int[] { -len, len, len }, new int[] { 0,
					len, -len }, 3);
			break;
		case TRIANGLE_LEFT:
			shape = new Polygon(new int[] { -len, len, -len }, new int[] { len,
					0, -len }, 3);
			break;
		case PLUS:
			shape = new Polygon(new int[] { 0, -len, len, 0, 0, 0 }, new int[] {
					0, 0, 0, 0, len, -len }, 6);
			break;
		case CROSS:
			shape = new Polygon(new int[] { 0, -len, len, 0, -len, len },
					new int[] { 0, -len, len, 0, len, -len }, 6);
			break;
		}

		return shape;
	}
}
