#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 964 $
$LastChangedDate:: 2019-07-03 08:21:59 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import numpy as np
import lcm.utils.pytperiodic as pm


def run_periodicMatrixRandom(nrows=2, ncols=3):
    """
    Example for PyTPeriodicMatrix
    using randomly generated signals with the defined size

    Parameters
    ----------
    nrows : int
        number of rows
    ncols : int
        number of cols

    """

    '''matrix 1'''
    xc_per = pm.PyTPeriodicMatrix(nrows, ncols, mode='random', maxorder=10)

    '''matrix 2'''
    zc_per = pm.PyTPeriodicMatrix(nrows, ncols, mode='random', maxorder=10)

    '''print'''
    print('xc_per', xc_per)
    print('zc_per', zc_per)

    '''plot with plotly'''
    xc_per.showFullPreviewPLY(mode='matrix')
    zc_per.showFullPreviewPLY(mode='matrix')


def run_periodicMatrixSine(nrows=2, ncols=3):
    """
    Example for PyTPeriodicMatrix
    using cosine and sine signals with the defined size

    Parameters
    ----------
    nrows : int
        number of rows
    ncols : int
        number of cols
    """

    '''input matrix from 1x1 periodics'''
    xc = np.zeros((nrows, ncols), dtype=np.object)
    for r in range(nrows):
        for c in range(ncols):
            magnitude = r + c
            xc[r, c] = pm.PyTPeriodicMatrix(1., magnitude, 0.)  # cosine

    '''output matrix from 1x1 periodics'''
    zc = np.zeros((nrows, ncols), dtype=np.object)
    for r in range(nrows):
        for c in range(ncols):
            magnitude = r + c
            zc[r, c] = pm.PyTPeriodicMatrix(1, magnitude, -np.pi / 2)  # sine

    '''numpy to periodic'''
    xc_per = pm.PyTPeriodicMatrix(xc, mode='objarray')
    zc_per = pm.PyTPeriodicMatrix(zc, mode='objarray')

    '''print'''
    print('xc_per', xc_per)
    print('zc_per', zc_per)

    '''plot with plotly'''
    xc_per.showFullPreviewPLY(mode='matrix')
    zc_per.showFullPreviewPLY(mode='matrix')


def run_periodicMatrixPlot(nrows=2, ncols=3):
    """
    Example for PyTPeriodicMatrix
    using randomly generated signals with the defined size

    Parameters
    ----------
    nrows : int
        number of rows
    ncols : int
        number of cols

    """

    '''matrix 1'''
    xc_per = pm.PyTPeriodicMatrix(nrows, ncols, mode='random', maxorder=10)

    '''matrix 2'''
    zc_per = pm.PyTPeriodicMatrix(nrows, ncols, mode='random', maxorder=10)

    '''print'''
    print('xc_per', xc_per)
    print('zc_per', zc_per)

    '''plot with plotly'''
    # xc_per.showPreviewPLY(mode='matrix', titles=[str(x) for x in range(nrows * ncols)])
    # zc_per.showPreviewPLY(mode='single', titles=['abc', ])
    zc_per.showFullPreviewPLY(mode='single', titles=['abc', ], orders=np.arange(0, 30, 1))
    # zc_per.showPreviewPLY(mode='single', titles=['a', ])
    # zc_per.showPreviewPLY(mode='row', titles=[str(x) for x in range(nrows+1)])


if __name__ == '__main__':
    # run_periodicMatrixRandom()
    # run_periodicMatrixSine()
    run_periodicMatrixPlot()
