/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.center;

import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.UIManager;
import javax.swing.WindowConstants;

import at.lcm.symspace.center.icons.sym.SyMIcons;
import at.lcm.symspace.center.views.CenterViewManager;
import at.lcm.symspace.gui.Gui;
import at.lcm.symspace.gui.Splash;

/**
 * Test Application for MagOpt
 * 
 * @author Werner Koppelsttter
 */
public class CenterGui extends Gui {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Gui-controller */
	private CenterController controller;

	/** Views manager */
	private CenterViewManager views;

	/**
	 * Start gui
	 * 
	 * @param args
	 *            Program-arguments
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
		}

		Splash splash = new Splash();
		splash.setVisible(true);
		CenterGui gui = new CenterGui();
		splash.setVisible(false);
		splash.dispose();
		gui.setVisible(true);

		if (args.length > 0) {
			gui.controller.mnu_file_open(new File(args[0]));
		}
	}

	/**
	 * Create gui
	 */
	public CenterGui() {
		super("SyMSpace Center");

		this.setIconImages(SyMIcons.getIcons());

		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {

			@Override
			public void windowClosing(WindowEvent e) {
				CenterGui.this.controller.mnu_file_exit();
			}
		});
		this.setLayout(new BorderLayout());

		// screen insets
		Insets insets = Toolkit.getDefaultToolkit()
				.getScreenInsets(GraphicsEnvironment
						.getLocalGraphicsEnvironment().getDefaultScreenDevice()
						.getDefaultConfiguration());
		// screen area
		Rectangle rect = GraphicsEnvironment.getLocalGraphicsEnvironment()
				.getDefaultScreenDevice().getDefaultConfiguration().getBounds();
		// working area
		Rectangle work = new Rectangle(rect.x + insets.left,
				rect.y + insets.top, rect.width - insets.left - insets.right,
				rect.height - insets.top - insets.bottom);

		this.setLocation(work.x + (work.width / 20),
				work.y + (work.height / 20));
		this.setSize((work.width * 9) / 10, (work.height * 9) / 10);

		this.toFront();

		this.controller = new CenterController(this);
		this.views = new CenterViewManager(this.controller);

		this.setJMenuBar(this.views.getMenu());

		this.add(this.views.getToolBar(), BorderLayout.PAGE_START);
		this.add(this.views.getStatusBar(), BorderLayout.PAGE_END);
		this.add(this.views.getComponent(), BorderLayout.CENTER);

		this.revalidate();
	}

	/**
	 * Get controller of GUI
	 * 
	 * @return Controller
	 */
	public CenterController getController() {
		return this.controller;
	}
}
