/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.center.icons;

import java.net.URL;

import at.lcm.bu21.magopt.structure.IParamCollector;
import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParamDirectory;
import at.lcm.bu21.magopt.structure.IParamField;
import at.lcm.bu21.magopt.structure.IParamFile;
import at.lcm.bu21.magopt.structure.IParamFormula;
import at.lcm.bu21.magopt.structure.IParamJavaFunction;
import at.lcm.bu21.magopt.structure.IParamLinkedField;
import at.lcm.bu21.magopt.structure.IParamMaterial;
import at.lcm.bu21.magopt.structure.IParamMatlabFunction;
import at.lcm.bu21.magopt.structure.IParamPythonFunction;
import at.lcm.bu21.magopt.structure.IParamPythonFunction_old;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;

/**
 * Parameter Icons
 * 
 * @author Werner Koppelsttter
 */
public class IconParameter {

	/** Package of icons */
	private static final String pack = "/" + IconParameter.class.getPackage().getName().replace('.', '/') + "/";

	/** Icon for parameter Project */
	public final static URL PROJECT = IconParameter.class.getResource(IconParameter.pack + "Project.png");
	/** Icon for parameter Container */
	public final static URL CONTAINER = IconParameter.class.getResource(IconParameter.pack + "ParamContainer.png");
	/** Icon for parameter JavaFunction */
	public final static URL JAVA = IconParameter.class.getResource(IconParameter.pack + "ParamJavaFunction.png");
	/** Icon for parameter MatlabFunction */
	public final static URL MATLAB = IconParameter.class.getResource(IconParameter.pack + "ParamMatlabFunction.png");
	/** Icon for parameter MatlabFunction */
	public final static URL PYTHON = IconParameter.class.getResource(IconParameter.pack + "ParamPythonFunction.png");
	/** Icon for parameter Material */
	public final static URL MATERIAL = IconParameter.class.getResource(IconParameter.pack + "ParamMaterial.png");
	/** Icon for parameter Material */
	public final static URL COLLECTOR = IconParameter.class.getResource(IconParameter.pack + "ParamCollector.png");
	/** Icon for parameter Field */
	public final static URL FIELD = IconParameter.class.getResource(IconParameter.pack + "ParamField.png");
	/** Icon for parameter Formula */
	public final static URL FORMULA = IconParameter.class.getResource(IconParameter.pack + "ParamFormula.png");
	/** Icon for parameter LinkedField */
	public final static URL LINKEDFIELD = IconParameter.class.getResource(IconParameter.pack + "ParamLinkedField.png");
	/** Icon for parameter File */
	public final static URL FILE = IconParameter.class.getResource(IconParameter.pack + "ParamFile.png");
	/** Icon for parameter Directory */
	public final static URL DIRECTORY = IconParameter.class.getResource(IconParameter.pack + "ParamDirectory.png");

	/**
	 * Get icon for selected parameter
	 * 
	 * @param param
	 *            Parameter
	 * @return Icon for parameter
	 */
	public static URL getIcon(IParameter param) {
		URL icon = null;

		if (param instanceof IProject) {
			icon = IconParameter.PROJECT;
		} else if (param instanceof IParamFormula) {
			icon = IconParameter.FORMULA;
		} else if (param instanceof IParamLinkedField) {
			icon = IconParameter.LINKEDFIELD;
		} else if (param instanceof IParamField) {
			icon = IconParameter.FIELD;
		} else if (param instanceof IParamMatlabFunction) {
			icon = IconParameter.MATLAB;
		} else if (param instanceof IParamPythonFunction) {
			icon = IconParameter.PYTHON;
		} else if (param instanceof IParamPythonFunction_old) {
			icon = IconParameter.PYTHON;
		} else if (param instanceof IParamJavaFunction) {
			icon = IconParameter.JAVA;
		} else if (param instanceof IParamMaterial) {
			icon = IconParameter.MATERIAL;
		} else if (param instanceof IParamCollector) {
			icon = IconParameter.COLLECTOR;
		} else if (param instanceof IParamContainer) {
			icon = IconParameter.CONTAINER;
		} else if (param instanceof IParamFile) {
			icon = IconParameter.FILE;
		} else if (param instanceof IParamDirectory) {
			icon = IconParameter.DIRECTORY;
		}

		return icon;
	}
}
