/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9858 $
 * $Date: 2020-05-29 15:28:25 +0200 (Fri, 29 May 2020) $
 */
package at.lcm.symspace.center.views;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.net.URL;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import at.lcm.bu21.magopt.structure.IParamCollector;
import at.lcm.bu21.magopt.structure.IParamComboBox;
import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParamField;
import at.lcm.bu21.magopt.structure.IParamFormula;
import at.lcm.bu21.magopt.structure.IParamFunction;
import at.lcm.bu21.magopt.structure.IParamLinkedField;
import at.lcm.bu21.magopt.structure.IParamMaterial;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;
import at.lcm.bu21.magopt.utils.MagOptConfig;
import at.lcm.symspace.center.CenterController;
import at.lcm.symspace.center.icons.IconMenu;
import at.lcm.symspace.center.icons.IconParameter;
import at.lcm.symspace.gui.utils.Storage;
import at.lcm.symspace.gui.views.MenuBar;

public class CenterMenuBar extends MenuBar implements ActionListener {

	// Actions
	private static String ACTION_NEW = "New";
	private static String ACTION_OPEN = "Open";
	private static String ACTION_CLOSE = "Close";
	private static String ACTION_SAVE = "Save";
	private static String ACTION_SAVE_AS = "Save as";
	private static String ACTION_SAVE_AS_TEMPLATE = "Save as template";
	private static String ACTION_OPEN_RECENT_FILE = "Open recent file";
	private static String ACTION_EXIT = "Exit";

	private static String ACTION_CUT_PARAMETER = "Cut parameter";
	private static String ACTION_COPY_PARAMETER = "Copy parameter";
	private static String ACTION_PASTE_PARAMETER = "Past parameter";
	private static String ACTION_DELETE_PARAMETER = "Delete parameter";
	private static String ACTION_DUPLICATE = "Duplicate";
	private static String ACTION_REDUCE = "Reduce";

	private static String ACTION_INSERT_COMPONENT = "Insert Component";
	private static String ACTION_INSERT_CONTAINER = "Insert Container";
	private static String ACTION_INSERT_FIELD = "Insert Field";
	private static String ACTION_INSERT_COMBOBOX = "Insert ComboBox";
	private static String ACTION_INSERT_FORMULA = "Insert Formula";
	private static String ACTION_INSERT_DIRECTORY = "Insert Directory";
	private static String ACTION_INSERT_FILE = "Insert File";
	private static String ACTION_INSERT_MATERIAL = "Insert Material";
	private static String ACTION_INSERT_FUNCTION = "Insert Function";
	private static String ACTION_INSERT_TEMPLATE = "Insert Template";

	private static String ACTION_CONVERT_CONTAINER = "Convert to container";
	private static String ACTION_CONVERT_FUNCTION = "Convert to function";
	private static String ACTION_CONVERT_UNLIKED_MATERIAL = "Convert to unlinked material";
	private static String ACTION_CONVERT_MATERIAL = "Convert to material";
	private static String ACTION_CONVERT_COLLECTOR = "Convert to collector";
	private static String ACTION_CONVERT_FIELD = "Convert to field";
	private static String ACTION_CONVERT_FORMULA = "Convert to formula";
	private static String ACTION_CONVERT_COMBOBOX = "Convert to combobox";
	private static String ACTION_CONVERT_LINKED_FIELD = "Convert to linked field";

	private static String ACTION_REFRESH = "Refresh";
	private static String ACTION_CHECK = "Check";
	private static String ACTION_REGENERATE = "Regenerate";
	private static String ACTION_JOBMANAGER = "Open JobManager";
	private static String ACTION_REGENERATE_ALL = "Regenerate all";
	private static String ACTION_MACRO = "Run macro";

	private static String ACTION_START_SIMULATION = "Start simulation";
	private static String ACTION_REMOTE_COMPUTE = "Remote compute";
	private static String ACTION_OPTIMIZE = "Optimize";

	private static String ACTION_CREATE_PDF = "Create pdf report";
	private static String ACTION_FILL_PDF = "Fill pdf report";
	private static String ACTION_CREATE_EXCEL = "Create Excel report";
	private static String ACTION_FILL_EXCEL = "Fill Excel report";

	private static String ACTION_CREATE_ALL_DOCFILES = "Create all DocFiles";
	private static String ACTION_CREATE_SELECTED_DOCFILES = "Create selected DocFiles";
	private static String ACTION_RESET_ALL_DOCFILES = "Reset all DocFiles";
	private static String ACTION_RESET_SELECTED_DOCFILES = "Reset selected DocFiles";
	private static String ACTION_DELETE_ALL_DOCFILES = "Delete all DocFiles";
	private static String ACTION_DELETE_SELECTED_DOCFILES = "Delete selected DocFiles";

	private static String ACTION_VARIANTS = "Variants";
	private static String ACTION_ANALYSIS = "Analysis";

	private static String ACTION_MAT_CHANGE_USER = "Change User";
	private static String ACTION_MAT_RESET_USER = "Reset User";
	private static String ACTION_MAT_USE_FILE = "Use Material File";
	private static String ACTION_MAT_CREATE_FILE = "Create Material File";

	private static String ACTION_EXTRACT_INDEX = "Extract Project Index";
	private static String ACTION_RESET_ALL = "Reset All Functions";
	private static String ACTION_RESET_SELECTED = "Reset Selected Function";
	private static String ACTION_UPDATE_SELECTED = "Update Selected Function";
	private static String ACTION_COMPARE_PROJECT = "Compare Project";

	private static String ACTION_PYTHON_ASSIGN_PROJECT = "Assign Project to Python";
	private static String ACTION_PYTHON_ASSIGN_PARAM = "Assign Parameter to Python";
	private static String ACTION_PYTHON_ASSIGN_DATA = "Assign Data to Python";
	private static String ACTION_PYTHON_GET_DATA = "Get Data from Python";
	private static String ACTION_PYTHON_WRITE_DATA = "Write Data to File";
	private static String ACTION_PYTHON_RESTART = "Restart python of project";
	private static String ACTION_PYTHON_SYSTEM = "Set python of project to console";

	private static String ACTION_MATLAB_ASSIGN_PROJECT = "Assign Project to Matlab";
	private static String ACTION_MATLAB_ASSIGN_PARAM = "Assign Parameter to Matlab";
	private static String ACTION_MATLAB_ASSIGN_DATA = "Assign Data to Matlab";
	private static String ACTION_MATLAB_GET_DATA = "Get Data from Matlab";

	private static String ACTION_HELP = "Help";
	private static String ACTION_HELP_UPDATE = "Help Update";
//	private static String ACTION_DOCU = "Docu";
//	private static String ACTION_DOCU_UPDATE = "Docu Update";
	private static String ACTION_ABOUT = "About";

	// KeyStrokes
	private static KeyStroke KEYSTROKE_NONE = null;

	private static KeyStroke KEYSTROKE_NEW = KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_OPEN = KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_SAVE = KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK);

	private static KeyStroke KEYSTROKE_INSERT_COMPONENT = KeyStroke.getKeyStroke(KeyEvent.VK_1,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_TEMPLATE = KeyStroke.getKeyStroke(KeyEvent.VK_2,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_FUNCTION = KeyStroke.getKeyStroke(KeyEvent.VK_3,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_CONTAINER = KeyStroke.getKeyStroke(KeyEvent.VK_4,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_FIELD = KeyStroke.getKeyStroke(KeyEvent.VK_5, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_FORMULA = KeyStroke.getKeyStroke(KeyEvent.VK_6,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_MATERIAL = KeyStroke.getKeyStroke(KeyEvent.VK_7,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_DIRECTORY = KeyStroke.getKeyStroke(KeyEvent.VK_8,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_INSERT_FILE = KeyStroke.getKeyStroke(KeyEvent.VK_9, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_CUT_PARAMETER = KeyStroke.getKeyStroke(KeyEvent.VK_X,
			InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK);
	private static KeyStroke KEYSTROKE_COPY_PARAMETER = KeyStroke.getKeyStroke(KeyEvent.VK_C,
			InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK);
	private static KeyStroke KEYSTROKE_PASTE_PARAMETER = KeyStroke.getKeyStroke(KeyEvent.VK_V,
			InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK);
	private static KeyStroke KEYSTROKE_DELETE_PARAMETER = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,
			InputEvent.CTRL_DOWN_MASK);

	private static KeyStroke KEYSTROKE_REFRESH = KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0);
	private static KeyStroke KEYSTROKE_REGENERATE = KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_JOBMANAGER = KeyStroke.getKeyStroke(KeyEvent.VK_J, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_REGENERATE_ALL = KeyStroke.getKeyStroke(KeyEvent.VK_G,
			InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK);
	private static KeyStroke KEYSTROKE_MACRO = KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_DOWN_MASK);

	private static KeyStroke KEYSTROKE_PYTHON_ASSIGN_PROJECT = KeyStroke.getKeyStroke(KeyEvent.VK_P,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_PYTHON_ASSIGN_OBJECT = KeyStroke.getKeyStroke(KeyEvent.VK_B,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_PYTHON_ASSIGN_DATA = KeyStroke.getKeyStroke(KeyEvent.VK_D,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_RESET_ALL = KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_RESET_SELECTED = KeyStroke.getKeyStroke(KeyEvent.VK_R,
			InputEvent.CTRL_DOWN_MASK);
	private static KeyStroke KEYSTROKE_UPDATE_SELECTED = KeyStroke.getKeyStroke(KeyEvent.VK_U,
			InputEvent.CTRL_DOWN_MASK);

	private static KeyStroke KEYSTROKE_HELP = KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0);

	CenterController controller;

	Map<JMenuItem, File> miRecentFiles;

	// Menu -> File
	JMenu mFile;
	JMenuItem miNew;
	JMenuItem miOpen;
	JMenuItem miClose;
	JMenuItem miSave;
	JMenuItem miSaveAs;
	JMenuItem miSaveAsTemplate;
	JMenuItem miExit;

	// Menu -> Edit
	// JMenuItem miCut;
	// JMenuItem miCopy;
	// JMenuItem miPaste;
	JMenuItem miCutParameter;
	JMenuItem miCopyParameter;
	JMenuItem miPasteParameter;
	JMenuItem miDeleteParameter;
	JMenu mConvert;
	JMenuItem miDuplicate;
	JMenuItem miReduce;

	// Menu -> Edit -> Insert
	JMenuItem miInsertComponent;
	JMenuItem miInsertContainer;
	JMenuItem miInsertField;
	JMenuItem miInsertComboBox;
	JMenuItem miInsertFormula;
	JMenuItem miInsertDirectory;
	JMenuItem miInsertFile;
	JMenuItem miInsertMaterial;
	JMenuItem miInsertFunction;
	JMenuItem miInsertTemplate;

	// Menu -> Edit -> Convert
	JMenuItem miConvertNotPossible;
	JMenuItem miConvert2Container;
	JMenuItem miConvert2Function;
	JMenuItem miConvert2UnlinkedMaterial;
	JMenuItem miConvert2Material;
	JMenuItem miConvert2Collector;
	JMenuItem miConvert2Field;
	JMenuItem miConvert2Formula;
	JMenuItem miConvert2Combobox;
	JMenuItem miConvert2LinkedField;

	// Menu -> Update
	JMenuItem miRefresh;
	JMenuItem miCheck;
	JMenuItem miRegenerate;
	JMenuItem miJobManager;
	JMenuItem miRegenerateAll;
	JMenuItem miMacro;

	// Menu -> SimulationSettings
	JCheckBoxMenuItem miStartSimulation;
	JCheckBoxMenuItem miRemoteCompute;
	JCheckBoxMenuItem miOptimize;

	// Menu -> Documentation
	JMenuItem miReportPdf;
	JMenuItem miFillPdf;
	JMenuItem miReportExcel;
	JMenuItem miFillExcel;

	// Menu -> Documentation -> DocFiles
	JMenuItem miDocGenerate;
	JMenuItem miDocGenerateSelected;
	JMenuItem miDocReset;
	JMenuItem miDocResetSelected;
	JMenuItem miDocDelete;
	JMenuItem miDocDeleteSelected;

	// Menu -> Extra
	JMenuItem miVariants;
	JMenuItem miAnalysis;

	// Menu -> Extra -> Material
	JMenuItem miMatChangeUser;
	JMenuItem miMatResetUser;
	JCheckBoxMenuItem miMatUseOffline;
	JMenuItem miMatCreateOffline;

	// Menu -> Debug
	JMenuItem miExtractIndex;
	JMenuItem miReset;
	JMenuItem miResetSelected;
	JMenuItem miUpdateSelected;
	JMenuItem miCompareProject;

	// Menu -> Debug -> Python
	JMenuItem miPyAssignProject;
	JMenuItem miPyAssignParam;
	JMenuItem miPyAssignData;
	JMenuItem miPyGetData;
	JMenuItem miPyWriteData;
	JMenuItem miPyRestart;
	JMenuItem miPySetSystem;

	// Menu -> Debug -> Matlab
	JMenuItem miMatlabAssignProject;
	JMenuItem miMatlabAssignParam;
	JMenuItem miMatlabAssignData;
	JMenuItem miMatlabGetData;

	// Menu -> Help
	JMenuItem miHelp;
	JMenuItem miHelpUpdate;
//	JMenuItem miDocu;
//	JMenuItem miDocuUpdate;
	JMenuItem miAbout;

	public CenterMenuBar(CenterController controller) {
		this.controller = controller;
		this.miRecentFiles = new Hashtable<JMenuItem, File>();

		this.add(this.createFileMenu());
		this.add(this.createEditMenu());
		this.add(this.createUpdateMenu());
		this.add(this.createSimulationSettingsMenu());
		this.add(this.createDocumentationMenu());
		this.add(this.createExtrasMenu());
		this.add(this.createWindowMenu());
		this.add(this.createDebugMenu());
		this.add(this.createHelpMenu());
	}

	private JMenu createFileMenu() {
		this.mFile = new JMenu("File");
		this.mFile.setMnemonic(KeyEvent.VK_F);
		this.mFile.addMenuListener(new MenuListener() {

			@Override
			public void menuSelected(MenuEvent e) {
				List<String> list;
				File file;
				JMenuItem item;
				int k = 1;
				int key;

				// remove items
				for (JMenuItem entry : CenterMenuBar.this.miRecentFiles.keySet()) {
					CenterMenuBar.this.mFile.remove(entry);
				}
				// add items
				list = new Storage().getRecentProjects();
				if (list != null) {
					for (String path : list) {
						file = new File(path);
						if (k < 10) {
							key = KeyEvent.VK_0 + k;
						} else {
							key = -1;
						}
						item = CenterMenuBar.this.makeMenuItem(
								k++ + " " + file.getName() + " (" + file.getParent() + ")", null,
								CenterMenuBar.ACTION_OPEN_RECENT_FILE, key, CenterMenuBar.KEYSTROKE_NONE,
								file.getAbsolutePath());
						CenterMenuBar.this.miRecentFiles.put(item, file);
						CenterMenuBar.this.mFile.insert(item, CenterMenuBar.this.mFile.getItemCount() - 2);
						item.setEnabled(CenterMenuBar.this.controller.getViewManager().isEnabled() && file.exists());
					}
				}
			}

			@Override
			public void menuDeselected(MenuEvent e) {
			}

			@Override
			public void menuCanceled(MenuEvent e) {
			}
		});

		this.miNew = this.mFile.add(this.makeMenuItem("New", IconMenu.NEW, CenterMenuBar.ACTION_NEW, KeyEvent.VK_N,
				CenterMenuBar.KEYSTROKE_NEW, "New project"));
		this.miOpen = this.mFile.add(this.makeMenuItem("Open ...", IconMenu.OPEN, CenterMenuBar.ACTION_OPEN,
				KeyEvent.VK_O, CenterMenuBar.KEYSTROKE_OPEN, "Open project"));
		this.miOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK));
		this.miClose = this.mFile.add(this.makeMenuItem("Close", null, CenterMenuBar.ACTION_CLOSE, KeyEvent.VK_C,
				CenterMenuBar.KEYSTROKE_NONE, "Close project"));
		// FIXME Close
		this.miClose.setVisible(false);

		this.mFile.addSeparator();

		this.miSave = this.mFile.add(this.makeMenuItem("Save", IconMenu.SAVE, CenterMenuBar.ACTION_SAVE, KeyEvent.VK_S,
				CenterMenuBar.KEYSTROKE_SAVE, "Save project"));
		this.miSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK));
		this.miSaveAs = this.mFile.add(this.makeMenuItem("Save as ...", null, CenterMenuBar.ACTION_SAVE_AS,
				KeyEvent.VK_A, CenterMenuBar.KEYSTROKE_NONE, "Save project as ..."));
		this.miSaveAs.setDisplayedMnemonicIndex(5);
		this.miSaveAsTemplate = this.mFile
				.add(this.makeMenuItem("Save as template ...", null, CenterMenuBar.ACTION_SAVE_AS_TEMPLATE,
						KeyEvent.VK_T, CenterMenuBar.KEYSTROKE_NONE, "Save project as template ..."));

		this.mFile.addSeparator();

		// recent Files

		this.mFile.addSeparator();

		this.miExit = this.mFile.add(this.makeMenuItem("Exit", null, CenterMenuBar.ACTION_EXIT, KeyEvent.VK_X,
				CenterMenuBar.KEYSTROKE_NONE, "Exit"));

		return this.mFile;
	}

	private JMenu createEditMenu() {
		JMenu menu;

		menu = new JMenu("Edit");
		menu.setMnemonic(KeyEvent.VK_E);

		menu.add(this.createEditInsertMenu());

		// menu.addSeparator();
		//
		// this.miCut = menu
		// .add(this.makeMenuItem("Cut", IconMenu.CUT, "Cut", "Cut"));
		// this.miCopy = menu
		// .add(this.makeMenuItem("Copy", IconMenu.COPY, "Copy", "Copy"));
		// this.miPaste = menu.add(
		// this.makeMenuItem("Pase", IconMenu.PASTE, "Paste", "Paste"));

		menu.addSeparator();

		this.miCutParameter = menu
				.add(this.makeMenuItem("Cut parameter", IconMenu.CUT, CenterMenuBar.ACTION_CUT_PARAMETER, KeyEvent.VK_T,
						CenterMenuBar.KEYSTROKE_CUT_PARAMETER, "Cut parameter"));
		this.miCopyParameter = menu
				.add(this.makeMenuItem("Copy parameter", IconMenu.COPY, CenterMenuBar.ACTION_COPY_PARAMETER,
						KeyEvent.VK_C, CenterMenuBar.KEYSTROKE_COPY_PARAMETER, "Copy parameter"));
		this.miPasteParameter = menu
				.add(this.makeMenuItem("Paste parameter", IconMenu.PASTE, CenterMenuBar.ACTION_PASTE_PARAMETER,
						KeyEvent.VK_P, CenterMenuBar.KEYSTROKE_PASTE_PARAMETER, "Paste parameter"));
		this.miDeleteParameter = menu
				.add(this.makeMenuItem("Delete parameter", IconMenu.DELETE, CenterMenuBar.ACTION_DELETE_PARAMETER,
						KeyEvent.VK_D, CenterMenuBar.KEYSTROKE_DELETE_PARAMETER, "Delete parameter"));

		menu.addSeparator();

		this.mConvert = this.createEditConvertMenu();
		menu.add(this.mConvert);

		menu.addSeparator();

		this.miDuplicate = menu.add(this.makeMenuItem("Duplicate index", IconMenu.DUPLICATE,
				CenterMenuBar.ACTION_DUPLICATE, KeyEvent.VK_U, CenterMenuBar.KEYSTROKE_NONE, "Duplicate index"));
		this.miReduce = menu.add(this.makeMenuItem("Remove index", IconMenu.REDUCE, CenterMenuBar.ACTION_REDUCE,
				KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_NONE, "Remove index"));

		return menu;
	}

	private JMenu createEditInsertMenu() {
		JMenu menu;

		menu = new JMenu("Insert");
		menu.setMnemonic(KeyEvent.VK_I);

		this.miInsertComponent = menu.add(this.makeMenuItem("Component", null, CenterMenuBar.ACTION_INSERT_COMPONENT,
				KeyEvent.VK_P, CenterMenuBar.KEYSTROKE_INSERT_COMPONENT, "Component"));

		this.miInsertTemplate = menu.add(this.makeMenuItem("Template", null, CenterMenuBar.ACTION_INSERT_TEMPLATE,
				KeyEvent.VK_T, CenterMenuBar.KEYSTROKE_INSERT_TEMPLATE, "Template"));

		menu.addSeparator();

		this.miInsertFunction = menu.add(this.makeMenuItem("Function", null, CenterMenuBar.ACTION_INSERT_FUNCTION,
				KeyEvent.VK_U, CenterMenuBar.KEYSTROKE_INSERT_FUNCTION, "Function"));

		this.miInsertContainer = menu
				.add(this.makeMenuItem("Container", IconParameter.CONTAINER, CenterMenuBar.ACTION_INSERT_CONTAINER,
						KeyEvent.VK_C, CenterMenuBar.KEYSTROKE_INSERT_CONTAINER, "Container"));
		this.miInsertField = menu.add(this.makeMenuItem("Field", IconParameter.FIELD, CenterMenuBar.ACTION_INSERT_FIELD,
				KeyEvent.VK_F, CenterMenuBar.KEYSTROKE_INSERT_FIELD, "Field"));
		this.miInsertFormula = menu.add(this.makeMenuItem("Formula", IconParameter.FORMULA,
				CenterMenuBar.ACTION_INSERT_FORMULA, KeyEvent.VK_O, CenterMenuBar.KEYSTROKE_INSERT_FORMULA, "Formula"));
		this.miInsertMaterial = menu
				.add(this.makeMenuItem("Material", IconParameter.MATERIAL, CenterMenuBar.ACTION_INSERT_MATERIAL,
						KeyEvent.VK_M, CenterMenuBar.KEYSTROKE_INSERT_MATERIAL, "Material"));
		this.miInsertDirectory = menu
				.add(this.makeMenuItem("Directory", IconParameter.DIRECTORY, CenterMenuBar.ACTION_INSERT_DIRECTORY,
						KeyEvent.VK_D, CenterMenuBar.KEYSTROKE_INSERT_DIRECTORY, "Directory"));
		this.miInsertFile = menu.add(this.makeMenuItem("File", IconParameter.FILE, CenterMenuBar.ACTION_INSERT_FILE,
				KeyEvent.VK_I, CenterMenuBar.KEYSTROKE_INSERT_FILE, "File"));
		// this.miInsertComboBox = menu.add(
		// this.makeMenuItem("ComboBox", null,
		// CenterMenuBar.ACTION_INSERT_COMBOBOX,
		// KeyEvent.VK_B, CenterMenuBars.KEYSTROKE_NONE, "ComboBox"));

		return menu;
	}

	private JMenu createEditConvertMenu() {
		JMenu menu;

		// TODO
		menu = new JMenu("Convert");
		menu.setMnemonic(KeyEvent.VK_O);
		// menu.addActionListener(this);

		this.miConvertNotPossible = menu.add(this.makeMenuItem("Not possible", null, "", KeyEvent.VK_X,
				CenterMenuBar.KEYSTROKE_NONE, "to container"));
		this.miConvertNotPossible.setEnabled(false);

		this.miConvert2Container = menu.add(this.makeMenuItem("to container", null,
				CenterMenuBar.ACTION_CONVERT_CONTAINER, KeyEvent.VK_C, CenterMenuBar.KEYSTROKE_NONE, "to container"));
		this.miConvert2Function = menu.add(this.makeMenuItem("to function", null, CenterMenuBar.ACTION_CONVERT_FUNCTION,
				KeyEvent.VK_U, CenterMenuBar.KEYSTROKE_NONE, "to function"));
		this.miConvert2UnlinkedMaterial = menu
				.add(this.makeMenuItem("to unlinked material", null, CenterMenuBar.ACTION_CONVERT_UNLIKED_MATERIAL,
						KeyEvent.VK_N, CenterMenuBar.KEYSTROKE_NONE, "to unlinked material"));
		this.miConvert2Material = menu.add(this.makeMenuItem("to material", null, CenterMenuBar.ACTION_CONVERT_MATERIAL,
				KeyEvent.VK_M, CenterMenuBar.KEYSTROKE_NONE, "to material"));
		this.miConvert2Collector = menu.add(this.makeMenuItem("to collector", null,
				CenterMenuBar.ACTION_CONVERT_COLLECTOR, KeyEvent.VK_M, CenterMenuBar.KEYSTROKE_NONE, "to collector"));
		this.miConvert2Field = menu.add(this.makeMenuItem("to field", null, CenterMenuBar.ACTION_CONVERT_FIELD,
				KeyEvent.VK_F, CenterMenuBar.KEYSTROKE_NONE, "to field"));
		this.miConvert2Formula = menu.add(this.makeMenuItem("to formula", null, CenterMenuBar.ACTION_CONVERT_FORMULA,
				KeyEvent.VK_O, CenterMenuBar.KEYSTROKE_NONE, "to formula"));
		this.miConvert2Combobox = menu.add(this.makeMenuItem("to combobox", null, CenterMenuBar.ACTION_CONVERT_COMBOBOX,
				KeyEvent.VK_B, CenterMenuBar.KEYSTROKE_NONE, "to combobox"));
		this.miConvert2LinkedField = menu
				.add(this.makeMenuItem("to linked field", null, CenterMenuBar.ACTION_CONVERT_LINKED_FIELD,
						KeyEvent.VK_L, CenterMenuBar.KEYSTROKE_NONE, "to linked field"));

		return menu;
	}

	private JMenu createUpdateMenu() {
		JMenu menu;

		menu = new JMenu("Update");
		menu.setMnemonic(KeyEvent.VK_U);

		this.miRefresh = menu.add(this.makeMenuItem("Refresh", IconMenu.REFRESH, CenterMenuBar.ACTION_REFRESH,
				KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_REFRESH, "Refresh"));
		this.miCheck = menu.add(this.makeMenuItem("Check", IconMenu.CHECK, CenterMenuBar.ACTION_REFRESH, KeyEvent.VK_C,
				CenterMenuBar.KEYSTROKE_NONE, "Check"));
		this.miRegenerate = menu
				.add(this.makeMenuItem("Regenerate", IconMenu.REGENERATE, CenterMenuBar.ACTION_REGENERATE,
						KeyEvent.VK_G, CenterMenuBar.KEYSTROKE_REGENERATE, "Regenerate one index"));

		menu.addSeparator();

		this.miJobManager = menu.add(this.makeMenuItem("JobManager", IconMenu.JOBMANAGER,
				CenterMenuBar.ACTION_JOBMANAGER, KeyEvent.VK_J, CenterMenuBar.KEYSTROKE_JOBMANAGER, "Open JobManager"));
		this.miRegenerateAll = menu.add(this.makeMenuItem("Regenerate all", null, CenterMenuBar.ACTION_REGENERATE_ALL,
				KeyEvent.VK_A, CenterMenuBar.KEYSTROKE_REGENERATE_ALL, "Regenerate all indizes"));
		this.miRegenerateAll.setDisplayedMnemonicIndex(11);

		menu.addSeparator();

		this.miMacro = menu.add(this.makeMenuItem("Run Macro", IconMenu.MACRO, CenterMenuBar.ACTION_MACRO,
				KeyEvent.VK_M, CenterMenuBar.KEYSTROKE_MACRO, "Run Macro"));

		return menu;
	}

	private JMenu createSimulationSettingsMenu() {
		JMenu menu;

		menu = new JMenu("Simulation Settings");
		menu.setMnemonic(KeyEvent.VK_S);

		this.miStartSimulation = this.makeCheckBoxMenuItem("Start Simulation", null,
				CenterMenuBar.ACTION_START_SIMULATION, KeyEvent.VK_S, CenterMenuBar.KEYSTROKE_NONE, "Start Simulation");
		menu.add(this.miStartSimulation);
		this.miRemoteCompute = this.makeCheckBoxMenuItem("Remote Compute", null, CenterMenuBar.ACTION_REMOTE_COMPUTE,
				KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_NONE, "Remote Compute");
		menu.add(this.miRemoteCompute);
		this.miOptimize = this.makeCheckBoxMenuItem("Optimize", null, CenterMenuBar.ACTION_OPTIMIZE, KeyEvent.VK_O,
				CenterMenuBar.KEYSTROKE_NONE, "Optimize");
		menu.add(this.miOptimize);

		return menu;
	}

	private JMenu createDocumentationMenu() {
		JMenu menu;

		menu = new JMenu("Documentation");
		menu.setMnemonic(KeyEvent.VK_D);

		// TODO
		this.miReportPdf = menu.add(this.makeMenuItem("Report: PDF", null, CenterMenuBar.ACTION_CREATE_PDF,
				KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_NONE, "Report: PDF"));
		this.miFillPdf = menu.add(this.makeMenuItem("Fill Report: PDF", null, CenterMenuBar.ACTION_FILL_PDF,
				KeyEvent.VK_F, CenterMenuBar.KEYSTROKE_NONE, "Fill Report: PDF"));

		menu.addSeparator();

		this.miReportExcel = menu.add(this.makeMenuItem("Report: Excel", null, CenterMenuBar.ACTION_CREATE_EXCEL,
				KeyEvent.VK_E, CenterMenuBar.KEYSTROKE_NONE, "Report: Excel"));
		this.miFillExcel = menu.add(this.makeMenuItem("Fill Report: Excel", null, CenterMenuBar.ACTION_FILL_EXCEL,
				KeyEvent.VK_I, CenterMenuBar.KEYSTROKE_NONE, "Fill Report: Excel"));

		menu.addSeparator();

		menu.add(this.createDocumentationDocFilesMenu());

		return menu;
	}

	private JMenu createDocumentationDocFilesMenu() {
		JMenu menu;

		menu = new JMenu("DocFiles");
		menu.setMnemonic(KeyEvent.VK_D);

		this.miDocGenerate = menu
				.add(this.makeMenuItem("Generate all DocFiles", null, CenterMenuBar.ACTION_CREATE_ALL_DOCFILES,
						KeyEvent.VK_G, CenterMenuBar.KEYSTROKE_NONE, "Generate all DocFiles"));
		this.miDocGenerateSelected = menu.add(
				this.makeMenuItem("Generate selected DocFiles", null, CenterMenuBar.ACTION_CREATE_SELECTED_DOCFILES,
						KeyEvent.VK_E, CenterMenuBar.KEYSTROKE_NONE, "Generate selected DocFiles"));

		menu.addSeparator();

		this.miDocReset = menu
				.add(this.makeMenuItem("Reset all DocFiles", null, CenterMenuBar.ACTION_RESET_ALL_DOCFILES,
						KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_NONE, "Reset all DocFiles"));
		this.miDocResetSelected = menu
				.add(this.makeMenuItem("Reset selected DocFiles", null, CenterMenuBar.ACTION_RESET_SELECTED_DOCFILES,
						KeyEvent.VK_S, CenterMenuBar.KEYSTROKE_NONE, "Reset selected DocFiles"));

		menu.addSeparator();

		this.miDocDelete = menu
				.add(this.makeMenuItem("Delete all DocFiles", null, CenterMenuBar.ACTION_DELETE_ALL_DOCFILES,
						KeyEvent.VK_D, CenterMenuBar.KEYSTROKE_NONE, "Delete all DocFiles"));
		this.miDocDeleteSelected = menu
				.add(this.makeMenuItem("Delete selected DocFiles", null, CenterMenuBar.ACTION_DELETE_SELECTED_DOCFILES,
						KeyEvent.VK_L, CenterMenuBar.KEYSTROKE_NONE, "Delete selected DocFiles"));

		return menu;
	}

	private JMenu createExtrasMenu() {
		JMenu menu;

		menu = new JMenu("Extras");
		menu.setMnemonic(KeyEvent.VK_X);

		menu.add(this.createExtrasMaterialMenu());

		this.miVariants = menu.add(this.makeMenuItem("Variants", null, CenterMenuBar.ACTION_VARIANTS, KeyEvent.VK_V,
				CenterMenuBar.KEYSTROKE_NONE, "Generate project variants"));
		this.miAnalysis = menu.add(this.makeMenuItem("Analysis", null, CenterMenuBar.ACTION_ANALYSIS, KeyEvent.VK_A,
				CenterMenuBar.KEYSTROKE_NONE, "Analyse project"));

		return menu;
	}

	private JMenu createExtrasMaterialMenu() {
		JMenu menu;

		menu = new JMenu("Material Database");
		menu.setMnemonic(KeyEvent.VK_M);

		this.miMatChangeUser = menu.add(this.makeMenuItem("Change User", null, CenterMenuBar.ACTION_MAT_CHANGE_USER,
				KeyEvent.VK_C, CenterMenuBar.KEYSTROKE_NONE, "Change material-database user"));
		this.miMatResetUser = menu.add(this.makeMenuItem("Reset User", null, CenterMenuBar.ACTION_MAT_RESET_USER,
				KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_NONE, "Reset material-database user to default"));
		this.miMatUseOffline = this.makeCheckBoxMenuItem("Use Material File (Offline)", null,
				CenterMenuBar.ACTION_MAT_USE_FILE, KeyEvent.VK_O, CenterMenuBar.KEYSTROKE_NONE,
				"Use Material File (Offline)");
		menu.add(this.miMatUseOffline);
		this.miMatCreateOffline = menu
				.add(this.makeMenuItem("Create Material File", null, CenterMenuBar.ACTION_MAT_CREATE_FILE,
						KeyEvent.VK_C, CenterMenuBar.KEYSTROKE_NONE, "Create Material File"));

		return menu;
	}

	private JMenu createWindowMenu() {
		JMenu menu;

		menu = new JMenu("Window");
		menu.setMnemonic(KeyEvent.VK_W);

		JMenu laf = (JMenu) menu.add(new JMenu("Style"));
		laf.setMnemonic(KeyEvent.VK_S);
		final ButtonGroup groupLaf = new ButtonGroup();
		// final JMenu theme = (JMenu) menu.add(new JMenu("Theme"));
		// final ButtonGroup groupTheme = new ButtonGroup();

		for (LookAndFeelInfo lafi : UIManager.getInstalledLookAndFeels()) {
			JRadioButtonMenuItem item = (JRadioButtonMenuItem) laf.add(new JRadioButtonMenuItem(lafi.getName()));
			item.setActionCommand(lafi.getClassName());
			groupLaf.add(item);
			if (lafi.getName().equals(UIManager.getLookAndFeel().getName())) {
				item.setSelected(true);
			}
			// FIXME
			item.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					try {
						JRadioButtonMenuItem mi = (JRadioButtonMenuItem) e.getSource();
						UIManager.setLookAndFeel(mi.getActionCommand());
						SwingUtilities.updateComponentTreeUI(CenterMenuBar.this.controller.getGuiComponent());
						// theme.setEnabled(mi.getText().equals("Metal"));
					} catch (Exception e1) {
						System.out.println(e1.getMessage());
						e1.printStackTrace();
					}
				}
			});
		}

		// JRadioButtonMenuItem def = (JRadioButtonMenuItem) theme
		// .add(new JRadioButtonMenuItem("DefaultMetalTheme"));
		// groupTheme.add(def);
		// def.addActionListener(new ActionListener() {
		//
		// @Override
		// public void actionPerformed(ActionEvent e) {
		// try {
		// JRadioButtonMenuItem mi = (JRadioButtonMenuItem) e
		// .getSource();
		// MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
		// SwingUtilities.updateComponentTreeUI(
		// Menu.this.controller.getGuiComponent());
		// } catch (Exception e1) {
		// System.out.println(e1.getMessage());
		// e1.printStackTrace();
		// }
		// }
		// });
		// JRadioButtonMenuItem ocean = (JRadioButtonMenuItem) theme
		// .add(new JRadioButtonMenuItem("OceanTheme"));
		// groupTheme.add(ocean);
		// ocean.addActionListener(new ActionListener() {
		// @Override
		// public void actionPerformed(ActionEvent e) {
		// try {
		// JRadioButtonMenuItem mi = (JRadioButtonMenuItem) e
		// .getSource();
		// MetalLookAndFeel.setCurrentTheme(new OceanTheme());
		// // if (Menu.this.views.getGui() != null) {
		// // SwingUtilities.updateComponentTreeUI(
		// // Menu.this.views.getGui());
		// // } else {
		// SwingUtilities.updateComponentTreeUI(
		// Menu.this.controller.getGuiComponent());
		// // }
		// } catch (Exception e1) {
		// System.out.println(e1.getMessage());
		// e1.printStackTrace();
		// }
		// }
		// });

		return menu;
	}

	private JMenu createDebugMenu() {
		JMenu menu;

		menu = new JMenu("Debug");
		menu.setMnemonic(KeyEvent.VK_G);

		menu.add(this.createDebugPythonMenu());
		menu.add(this.createDebugMatlabMenu());

		// FIXME IPython

		menu.addSeparator();

		this.miExtractIndex = menu
				.add(this.makeMenuItem("Extract project index", null, CenterMenuBar.ACTION_EXTRACT_INDEX, KeyEvent.VK_X,
						CenterMenuBar.KEYSTROKE_NONE, "Extract current index to new project"));

		menu.addSeparator();

		this.miReset = menu.add(this.makeMenuItem("Reset All Functions", null, CenterMenuBar.ACTION_RESET_ALL,
				KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_RESET_ALL, "Reset All Functions"));
		this.miResetSelected = menu
				.add(this.makeMenuItem("Reset Selected Function", null, CenterMenuBar.ACTION_RESET_SELECTED,
						KeyEvent.VK_S, CenterMenuBar.KEYSTROKE_RESET_SELECTED, "Reset Selected Function"));
		this.miUpdateSelected = menu
				.add(this.makeMenuItem("Update Selected Function", null, CenterMenuBar.ACTION_UPDATE_SELECTED,
						KeyEvent.VK_U, CenterMenuBar.KEYSTROKE_UPDATE_SELECTED, "Update Selected Function"));

		menu.addSeparator();

		this.miCompareProject = menu.add(this.makeMenuItem("Compare Project", null,
				CenterMenuBar.ACTION_COMPARE_PROJECT, KeyEvent.VK_C, CenterMenuBar.KEYSTROKE_NONE, "Compare Project"));

		return menu;
	}

	private JMenu createDebugPythonMenu() {
		JMenu menu;

		menu = new JMenu("Python");
		menu.setMnemonic(KeyEvent.VK_P);

		this.miPyAssignProject = menu.add(this.makeMenuItem("Assign Project Handle in Python Console", null,
				CenterMenuBar.ACTION_PYTHON_ASSIGN_PROJECT, KeyEvent.VK_P,
				CenterMenuBar.KEYSTROKE_PYTHON_ASSIGN_PROJECT, "Assign Project Handle in Python Console"));
		this.miPyAssignParam = menu.add(this.makeMenuItem("Assign Selected Object Handle in Python Console", null,
				CenterMenuBar.ACTION_PYTHON_ASSIGN_PARAM, KeyEvent.VK_S, CenterMenuBar.KEYSTROKE_PYTHON_ASSIGN_OBJECT,
				"Assign Selected Object Handle in Python Console"));
		this.miPyAssignParam.setDisplayedMnemonicIndex(7);
		this.miPyAssignData = menu.add(this.makeMenuItem("Assign Data of Selected Object in Python Console", null,
				CenterMenuBar.ACTION_PYTHON_ASSIGN_DATA, KeyEvent.VK_D, CenterMenuBar.KEYSTROKE_PYTHON_ASSIGN_DATA,
				"Assign Data of Selected Object in Python Console"));
		this.miPyGetData = menu.add(this.makeMenuItem("Get Data from Python Console to Selected Object", null,
				CenterMenuBar.ACTION_PYTHON_GET_DATA, KeyEvent.VK_G, CenterMenuBar.KEYSTROKE_NONE,
				"Get Data from Python Console to Selected Object"));
		this.miPyWriteData = menu.add(this.makeMenuItem("Write Data of Selected Object to Pickle-File", null,
				CenterMenuBar.ACTION_PYTHON_WRITE_DATA, KeyEvent.VK_F, CenterMenuBar.KEYSTROKE_NONE,
				"Write Data of Selected Object to Pickle-File"));

		menu.addSeparator();

		this.miPyRestart = menu
				.add(this.makeMenuItem("Restart python of project", null, CenterMenuBar.ACTION_PYTHON_RESTART,
						KeyEvent.VK_R, CenterMenuBar.KEYSTROKE_NONE, "Restart pythonof project"));
		this.miPySetSystem = menu
				.add(this.makeMenuItem("Set jep to python console", null, CenterMenuBar.ACTION_PYTHON_SYSTEM,
						KeyEvent.VK_J, CenterMenuBar.KEYSTROKE_NONE, "Set jep to python console"));

		return menu;
	}

	private JMenu createDebugMatlabMenu() {
		JMenu menu;

		menu = new JMenu("Matlab");
		menu.setMnemonic(KeyEvent.VK_M);

		this.miMatlabAssignProject = menu.add(this.makeMenuItem("Assign Project Handle in Workspace", null,
				CenterMenuBar.ACTION_MATLAB_ASSIGN_PROJECT, KeyEvent.VK_P, CenterMenuBar.KEYSTROKE_NONE,
				"Assign Project Handle in Workspace"));
		this.miMatlabAssignParam = menu.add(this.makeMenuItem("Assign Selected Object Handle in Workspace", null,
				CenterMenuBar.ACTION_MATLAB_ASSIGN_PARAM, KeyEvent.VK_S, CenterMenuBar.KEYSTROKE_NONE,
				"Assign Selected Object Handle in Workspace"));
		this.miMatlabAssignParam.setDisplayedMnemonicIndex(7);
		this.miMatlabAssignData = menu.add(this.makeMenuItem("Assign Data of Selected Object in Workspace", null,
				CenterMenuBar.ACTION_MATLAB_ASSIGN_DATA, KeyEvent.VK_D, CenterMenuBar.KEYSTROKE_NONE,
				"Assign Data of Selected Object in Workspace"));
		this.miMatlabGetData = menu.add(this.makeMenuItem("Get Data from Matlab workspace to Selected Object", null,
				CenterMenuBar.ACTION_MATLAB_GET_DATA, KeyEvent.VK_G, CenterMenuBar.KEYSTROKE_NONE,
				"Get Data from Matlab workspace to Selected Object"));

		return menu;
	}

	private JMenu createHelpMenu() {
		JMenu menu;

		menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);

		this.miHelp = menu.add(this.makeMenuItem("Help", null, CenterMenuBar.ACTION_HELP, KeyEvent.VK_H,
				CenterMenuBar.KEYSTROKE_HELP, "Help"));
		this.miHelpUpdate = menu.add(this.makeMenuItem("Update Help", null, CenterMenuBar.ACTION_HELP_UPDATE,
				KeyEvent.VK_U, CenterMenuBar.KEYSTROKE_NONE, "Update Help"));

		menu.addSeparator();

//		this.miDocu = menu.add(this.makeMenuItem("Documentation", null, CenterMenuBar.ACTION_DOCU, KeyEvent.VK_D,
//				CenterMenuBar.KEYSTROKE_NONE, "Documetation"));
//		this.miDocuUpdate = menu.add(this.makeMenuItem("Update Documentation", null, CenterMenuBar.ACTION_DOCU_UPDATE,
//				KeyEvent.VK_T, CenterMenuBar.KEYSTROKE_NONE, "Update Documetation"));
//
//		menu.addSeparator();

		this.miAbout = menu.add(this.makeMenuItem("About SyMSpace Center", null, CenterMenuBar.ACTION_ABOUT,
				KeyEvent.VK_A, CenterMenuBar.KEYSTROKE_NONE, "About SyMSpace Center"));

		return menu;
	}

	private JMenuItem makeMenuItem(String name, URL icon, String actionCommand, int mnemonic, KeyStroke stroke,
			String toolTipText) {
		JMenuItem item;

		if (icon != null) {
			try {
				item = new JMenuItem(name, new ImageIcon(icon));
			} catch (Exception e) {
				item = new JMenuItem(name);
			}
		} else {
			item = new JMenuItem(name);
		}
		item.setEnabled(false);
		item.setActionCommand(actionCommand);
		if (mnemonic >= 0) {
			item.setMnemonic(mnemonic);
		}
		if (stroke != null) {
			item.setAccelerator(stroke);
		}
		item.setToolTipText(toolTipText);
		item.addActionListener(this);

		return item;
	}

	private JCheckBoxMenuItem makeCheckBoxMenuItem(String name, URL icon, String actionCommand, int mnemonic,
			KeyStroke stroke, String toolTipText) {
		JCheckBoxMenuItem item;

		if (icon != null) {
			try {
				item = new JCheckBoxMenuItem(name, new ImageIcon(icon), false);
			} catch (Exception e) {
				item = new JCheckBoxMenuItem(name, null, false);
			}
		} else {
			item = new JCheckBoxMenuItem(name, null, false);

		}
		item.setActionCommand(actionCommand);
		item.setMnemonic(mnemonic);
		if (stroke != null) {
			item.setAccelerator(stroke);
		}
		item.setToolTipText(toolTipText);
		item.addActionListener(this);

		return item;

	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// File
		if (e.getActionCommand() == CenterMenuBar.ACTION_NEW) {
			this.controller.mnu_file_new();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_OPEN) {
			this.controller.mnu_file_open();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CLOSE) {
			this.controller.mnu_file_close();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_SAVE) {
			this.controller.mnu_file_save();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_SAVE_AS) {
			this.controller.mnu_file_save_as();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_SAVE_AS_TEMPLATE) {
			this.controller.mnu_file_save_as_template();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_OPEN_RECENT_FILE) {
			this.controller.mnu_file_open(this.miRecentFiles.get(e.getSource()));
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_EXIT) {
			this.controller.mnu_file_exit();

			// Edit insert
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_COMPONENT) {
			this.controller.mnu_edit_insert_component();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_TEMPLATE) {
			this.controller.mnu_edit_insert_template();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_FUNCTION) {
			this.controller.mnu_edit_insert_function();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_CONTAINER) {
			this.controller.mnu_edit_insert_container();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_FIELD) {
			this.controller.mnu_edit_insert_field();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_FORMULA) {
			this.controller.mnu_edit_insert_formula();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_MATERIAL) {
			this.controller.mnu_edit_insert_material();
//		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_COLLECTOR) {
//			this.controller.mnu_edit_insert_collector();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_DIRECTORY) {
			this.controller.mnu_edit_insert_directory();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_INSERT_FILE) {
			this.controller.mnu_edit_insert_file();
			// } else if (e.getActionCommand() == Menu.ACTION_INSERT_COMBOBOX) {
			// this.controller.mnu_edit_insert_combobox();

			// Edit
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CUT_PARAMETER) {
			this.controller.mnu_edit_cut_parameter();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_COPY_PARAMETER) {
			this.controller.mnu_edit_copy_parameter();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PASTE_PARAMETER) {
			this.controller.mnu_edit_paste_parameter();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_DELETE_PARAMETER) {
			this.controller.mnu_edit_delete_parameter();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_DUPLICATE) {
			this.controller.mnu_edit_duplicate();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_REDUCE) {
			this.controller.mnu_edit_reduce();

			// Edit convert
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_CONTAINER) {
			this.controller.mnu_edit_convert_to_container();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_FUNCTION) {
			this.controller.mnu_edit_convert_to_function();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_UNLIKED_MATERIAL) {
			this.controller.mnu_edit_convert_to_unlinked_material();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_MATERIAL) {
			this.controller.mnu_edit_convert_to_material();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_COLLECTOR) {
			this.controller.mnu_edit_convert_to_collector();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_FIELD) {
			this.controller.mnu_edit_convert_to_field();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_FORMULA) {
			this.controller.mnu_edit_convert_to_formula();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_COMBOBOX) {
			this.controller.mnu_edit_convert_to_combobox();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CONVERT_LINKED_FIELD) {
			this.controller.mnu_edit_convert_to_linked_field();

			// Update
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_REFRESH) {
			this.controller.mnu_update_refresh();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CHECK) {
			this.controller.mnu_update_check();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_REGENERATE) {
			this.controller.mnu_update_regenerate();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_JOBMANAGER) {
			this.controller.mnu_update_jobmanager();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_REGENERATE_ALL) {
			this.controller.mnu_update_regenerate_all();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MACRO) {
			this.controller.mnu_update_macro();

			// Simulation settings
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_START_SIMULATION) {
			this.controller.mnu_simulation_start();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_REMOTE_COMPUTE) {
			this.controller.mnu_simulation_remote();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_OPTIMIZE) {
			this.controller.mnu_simulation_optimize();

			// Documentation
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CREATE_PDF) {
			this.controller.mnu_documentation_create_pdf_report();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_FILL_PDF) {
			this.controller.mnu_documentation_fill_pdf_report();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CREATE_EXCEL) {
			this.controller.mnu_documentation_create_excel_report();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_FILL_EXCEL) {
			this.controller.mnu_documentation_fill_excel_report();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CREATE_ALL_DOCFILES) {
			this.controller.mnu_documentation_docfile_generate_all();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_CREATE_SELECTED_DOCFILES) {
			this.controller.mnu_documentation_docfile_generate_selected();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_RESET_ALL_DOCFILES) {
			this.controller.mnu_documentation_docfile_reset_all();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_RESET_SELECTED_DOCFILES) {
			this.controller.mnu_documentation_docfile_reset_selected();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_DELETE_ALL_DOCFILES) {
			this.controller.mnu_documentation_docfile_delete_all();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_DELETE_SELECTED_DOCFILES) {
			this.controller.mnu_documentation_docfile_delete_selected();

			// Extra
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MAT_CHANGE_USER) {
			this.controller.mnu_extra_material_change_user();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MAT_RESET_USER) {
			this.controller.mnu_extra_material_reset_user();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MAT_USE_FILE) {
			this.controller.mnu_extra_material_use_file();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MAT_CREATE_FILE) {
			this.controller.mnu_extra_material_create_file();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_VARIANTS) {
			this.controller.mnu_extra_variants();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_ANALYSIS) {
			this.controller.mnu_extra_analysis();

			// Debug
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_ASSIGN_PROJECT) {
			this.controller.mnu_debug_python_assign_project();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_ASSIGN_PARAM) {
			this.controller.mnu_debug_python_assign_param();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_ASSIGN_DATA) {
			this.controller.mnu_debug_python_assign_data();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_GET_DATA) {
			this.controller.mnu_debug_python_get_data();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_WRITE_DATA) {
			this.controller.mnu_debug_python_write_data();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_RESTART) {
			this.controller.mnu_debug_python_restart();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_PYTHON_SYSTEM) {
			this.controller.mnu_debug_python_assign_system_jep();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MATLAB_ASSIGN_PROJECT) {
			this.controller.mnu_debug_matlab_assign_project();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MATLAB_ASSIGN_PARAM) {
			this.controller.mnu_debug_matlab_assign_param();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MATLAB_ASSIGN_DATA) {
			this.controller.mnu_debug_matlab_assign_data();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_MATLAB_GET_DATA) {
			this.controller.mnu_debug_matlab_get_data();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_EXTRACT_INDEX) {
			this.controller.mnu_debug_extract_index();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_RESET_ALL) {
			this.controller.mnu_debug_reset_all();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_RESET_SELECTED) {
			this.controller.mnu_debug_reset_selected();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_UPDATE_SELECTED) {
			this.controller.mnu_debug_update_selected();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_COMPARE_PROJECT) {
			this.controller.mnu_debug_compare_project();

			// Help
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_HELP) {
			this.controller.mnu_help_help();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_HELP_UPDATE) {
			this.controller.mnu_help_help_update();
//		} else if (e.getActionCommand() == CenterMenuBar.ACTION_DOCU) {
//			this.controller.mnu_help_docu();
//		} else if (e.getActionCommand() == CenterMenuBar.ACTION_DOCU_UPDATE) {
//			this.controller.mnu_help_docu_update();
		} else if (e.getActionCommand() == CenterMenuBar.ACTION_ABOUT) {
			this.controller.mnu_help_about();
		} else {
			System.out.println("Not implemented");
		}

	}

	@Override
	public void refresh() {
		boolean enable = this.isEnabled();
		boolean projectExist = enable && (this.controller.getProject() != null);
		IParameter sel = this.controller.getSelected();
		boolean isOfflineMaterial = this.controller.isOfflineMaterial();
		boolean debug = false;
		String str;

		str = MagOptConfig.getConfig("CenterDebug");
		if (str != null) {
			try {
				debug = Boolean.parseBoolean(str);
			} catch (Throwable ex) {
			}
		}

		// FIXME dirty

		// Menu -> File
		this.miNew.setEnabled(enable);
		this.miOpen.setEnabled(enable);
		this.miClose.setEnabled(projectExist);
		this.miSave.setEnabled(projectExist);
		this.miSaveAs.setEnabled(projectExist);
		this.miSaveAsTemplate.setEnabled(projectExist);
		this.miExit.setEnabled(true);

		// Menu -> Edit
		// TODO
		// this.miCut.setEnabled(projectExist);
		// this.miCopy.setEnabled(projectExist);
		// this.miPaste.setEnabled(projectExist);
		this.miCutParameter.setEnabled(projectExist && !(sel instanceof IProject));
		this.miCopyParameter.setEnabled(projectExist);
		this.miPasteParameter.setEnabled(projectExist);
		this.miDeleteParameter.setEnabled(projectExist && !(sel instanceof IProject));
		this.miDuplicate.setEnabled(projectExist);
		this.miReduce.setEnabled(projectExist);

		// Menu -> Edit -> Insert
		this.miInsertComponent.setEnabled(projectExist);
		this.miInsertContainer.setEnabled(projectExist);
		this.miInsertField.setEnabled(projectExist);
		// this.miInsertComboBox.setEnabled(projectExist);
		this.miInsertFormula.setEnabled(projectExist);
		this.miInsertDirectory.setEnabled(projectExist);
		this.miInsertFile.setEnabled(projectExist);
		this.miInsertMaterial.setEnabled(projectExist);
		this.miInsertFunction.setEnabled(projectExist);
		this.miInsertTemplate.setEnabled(projectExist);

		// Menu -> Edit -> Convert
		this.miConvert2Container.setEnabled(enable);
		this.miConvert2Function.setEnabled(enable);
		this.miConvert2UnlinkedMaterial.setEnabled(enable);
		this.miConvert2Material.setEnabled(enable);
		this.miConvert2Collector.setEnabled(enable);
		this.miConvert2Field.setEnabled(enable);
		this.miConvert2Formula.setEnabled(enable);
		this.miConvert2Combobox.setEnabled(enable);
		this.miConvert2LinkedField.setEnabled(enable);

		this.miConvertNotPossible.setVisible(false);
		this.miConvert2Container.setVisible(false);
		this.miConvert2Function.setVisible(false);
		this.miConvert2UnlinkedMaterial.setVisible(false);
		this.miConvert2Material.setVisible(false);
		this.miConvert2Collector.setVisible(false);
		this.miConvert2Field.setVisible(false);
		this.miConvert2Formula.setVisible(false);
		this.miConvert2Combobox.setVisible(false);
		this.miConvert2LinkedField.setVisible(false);
		if (sel != null) {
			if (sel instanceof IProject) {
				this.miConvertNotPossible.setVisible(true);
			} else if (sel instanceof IParamFunction) {
				this.miConvert2Container.setVisible(projectExist);
				this.miConvert2Function.setVisible(projectExist);
			} else if (sel instanceof IParamMaterial) {
				this.miConvert2Container.setVisible(projectExist);
				this.miConvert2UnlinkedMaterial.setVisible(projectExist);
			} else if (sel instanceof IParamMaterial) {
				this.miConvert2Container.setVisible(projectExist);
			} else if (sel instanceof IParamCollector) {
				this.miConvert2Container.setVisible(projectExist);
			} else if (sel instanceof IParamContainer) {
				this.miConvert2Function.setVisible(projectExist);
				this.miConvert2Material.setVisible(projectExist);
				// FIXME File-Version
//				this.miConvert2Collector.setVisible(projectExist);
			} else if (sel instanceof IParamFormula) {
				this.miConvert2Field.setVisible(projectExist);
			} else if (sel instanceof IParamLinkedField) {
				this.miConvert2Field.setVisible(projectExist);
			} else if (sel instanceof IParamComboBox) {
				this.miConvert2Field.setVisible(projectExist);
			} else if (sel instanceof IParamField) {
				this.miConvert2Formula.setVisible(projectExist);
				this.miConvert2Combobox.setVisible(projectExist);
				this.miConvert2LinkedField.setVisible(projectExist);
			} else {
				this.miConvertNotPossible.setVisible(true);
			}
		}
		// FIXME

		// Menu -> Update
		this.miRefresh.setEnabled(enable);
		this.miCheck.setEnabled(projectExist);
		this.miRegenerate.setEnabled(projectExist);
		this.miJobManager.setEnabled(true);
		this.miRegenerateAll.setEnabled(projectExist);
		this.miMacro.setEnabled(enable);

		// Menu -> SimulationSettings
		this.miStartSimulation.setSelected(this.controller.isStartSimulation());
		this.miRemoteCompute.setSelected(this.controller.isRemoteCompute());
		this.miOptimize.setSelected(this.controller.isOptimize());

		// Menu -> Documentation
		this.miReportPdf.setEnabled(projectExist);
		this.miFillPdf.setEnabled(projectExist);
		this.miReportExcel.setEnabled(projectExist);
		this.miFillExcel.setEnabled(projectExist);

		// Menu -> Documentation -> DocFiles
		this.miDocGenerate.setEnabled(projectExist);
		this.miDocGenerateSelected.setEnabled(projectExist);
		this.miDocReset.setEnabled(projectExist);
		this.miDocResetSelected.setEnabled(projectExist);
		this.miDocDelete.setEnabled(projectExist);
		this.miDocDeleteSelected.setEnabled(projectExist);

		// Menu -> Extra
		this.miVariants.setEnabled(projectExist);
		this.miAnalysis.setEnabled(projectExist);

		// Menu -> Extra -> Material
		this.miMatChangeUser.setEnabled(!isOfflineMaterial);
		this.miMatResetUser.setEnabled(!isOfflineMaterial);
		this.miMatUseOffline.setEnabled(true);
		this.miMatUseOffline.setSelected(isOfflineMaterial);
		this.miMatCreateOffline.setEnabled(!isOfflineMaterial);

		// Menu -> Window

		// Menu -> Debug
		this.miExtractIndex.setEnabled(projectExist);
		this.miReset.setEnabled(projectExist);
		this.miResetSelected.setEnabled(projectExist);
		this.miUpdateSelected.setEnabled(projectExist);
		this.miCompareProject.setEnabled(true);

		// Menu -> Debug -> Python
		this.miPyAssignProject.setEnabled(projectExist);
		this.miPyAssignParam.setEnabled(projectExist);
		this.miPyAssignData.setEnabled(projectExist);
		this.miPyGetData.setEnabled(projectExist);
		this.miPyWriteData.setEnabled(projectExist);
		this.miPyRestart.setEnabled(projectExist);
		this.miPySetSystem.setEnabled(projectExist);
		this.miPySetSystem.setVisible(debug);

		// Menu -> Debug -> Matlab
		this.miMatlabAssignProject.setEnabled(projectExist);
		this.miMatlabAssignParam.setEnabled(projectExist);
		this.miMatlabAssignData.setEnabled(projectExist);
		this.miMatlabGetData.setEnabled(projectExist);

		// Menu -> Help
		this.miHelp.setEnabled(enable && this.controller.exist_help());
		this.miHelpUpdate.setEnabled(enable);
//		this.miDocu.setEnabled(enable && this.controller.exist_docu());
//		this.miDocuUpdate.setEnabled(enable);
		this.miAbout.setEnabled(true);
	}

	@Override
	public void setEnabled(boolean aFlag) {
		super.setEnabled(aFlag);
		this.refresh();
	}
}
