/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Hashtable;
import java.util.Map;

import at.lcm.bu21.general.dtypes.TData;
import jobmanager.JobInterface;

/**
 * Run a function
 * 
 * @author Werner Koppelsttter
 */
public class CallFunctionJob implements JobInterface {

	/**
	 * Run a function
	 * 
	 * @param args
	 *            Command-line parameters
	 * @exception Exception
	 */
	@Override
	public void runJob(String... args) throws Exception {
		FileInputStream in;
		FileOutputStream out;
		Map<String, TData> map;
		Map<String, Object> arguments = new Hashtable<String, Object>();
		CallFunction obj;
		int status;

		// temporary Folder
		arguments.put("TmpFolder", new File("").getAbsoluteFile());

		// input parameter
		in = new FileInputStream(new File(args[1]).getAbsoluteFile());
		map = TData.readMap(in);
		in.close();

		// run
		obj = (CallFunction) Class.forName(args[0]).getConstructor().newInstance();
		status = obj.run(arguments, map);

		// output parameter
		if (status == 0) {
			out = new FileOutputStream(args[2]);
			TData.writeMap(out, map);
			out.close();
		}
	}

}
