/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.io.File;
import java.rmi.RemoteException;

/**
 * Interface of directory
 * 
 * @author Werner Koppelsttter
 */
public interface IParamDirectory extends IParameter {

	/**
	 * Get reference location of directory
	 * 
	 * @return Location
	 * @throws RemoteException
	 */
	public Location getLocation() throws RemoteException;

	/**
	 * Change reference location of directory
	 * 
	 * @param location
	 * @throws RemoteException
	 */
	public void changeLocation(Location location) throws RemoteException;

	/**
	 * Get directory of current dataset
	 * 
	 * @return Directory
	 * @throws RemoteException
	 */
	public String getRelDirPath() throws RemoteException;

	/**
	 * Set relative path of current dataset
	 * 
	 * @param relPath
	 *            Relative path
	 * @throws RemoteException
	 */
	public void setRelDirPath(String relPath) throws RemoteException;

	/**
	 * Get absolute-path of directory
	 * 
	 * @return absolute-path
	 * @throws RemoteException
	 */
	public File getAbsoluteDirectory() throws RemoteException;

	/**
	 * Set absolute-path to directory
	 * 
	 * @param dir
	 *            File with absolute path
	 * @throws RemoteException
	 */
	public void setAbsoluteDirectory(File dir) throws RemoteException;

	/**
	 * Show directory
	 * 
	 * @throws RemoteException
	 */
	public void show() throws RemoteException;

	@Override
	public IParamDirectory getClientImpl() throws RemoteException;
}
