/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.client;

import java.rmi.RemoteException;
import java.util.Map;

import magopt.structure.database.DBItem;
import magopt.structure.database.MaterialConnection;
import at.lcm.bu21.magopt.structure.IDepend;
import at.lcm.bu21.magopt.structure.IParamMaterial;

/**
 * Client adapter for material interface
 * 
 * @author Werner Koppelsttter
 */
public class CParamMaterial extends CParamContainer implements IParamMaterial {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Material class */
	private IParamMaterial param;

	/**
	 * Create client adapter for material interface
	 * 
	 * @param param
	 *            Material
	 */
	public CParamMaterial(IParamMaterial param) {
		super(param);
		this.param = param;
	}

	@Override
	public IDepend getDepend() throws RemoteException {
		return this.param.getDepend().getClientImpl();
	}

	@Override
	public boolean getUpdateRequired() throws RemoteException {
		return this.param.getUpdateRequired();
	}

	@Override
	public boolean getUpdateRequired(int index) throws RemoteException {
		return this.param.getUpdateRequired(index);
	}

	@Override
	public void setUpdateRequired(boolean update) throws RemoteException {
		this.param.setUpdateRequired(update);
	}

	@Override
	public void setUpdateRequired(int index, boolean update)
			throws RemoteException {
		this.param.setUpdateRequired(index, update);
	}

	@Override
	public String getMaterialClassName() throws RemoteException {
		return this.param.getMaterialClassName();
	}

	@Override
	public String getMaterialName() throws RemoteException {
		return this.param.getMaterialName();
	}

	@Override
	public String getDatasetName() throws RemoteException {
		return this.param.getDatasetName();
	}

	@Override
	public long getDatasetID() throws RemoteException {
		return this.param.getDatasetID();
	}

	@Override
	public void setDatasetID(DBItem dataset, MaterialConnection conn)
			throws RemoteException {
		this.param.setDatasetID(dataset, conn);
	}

	@Override
	public void setDatasetID(long datasetID, MaterialConnection conn)
			throws RemoteException {
		this.param.setDatasetID(datasetID, conn);
	}

	@Override
	public void reset() throws RemoteException {
		this.param.reset();
	}

	@Override
	public int update(Map<String, Object> args) throws Exception,
			RemoteException {
		return this.param.update(args);
	}

	@Override
	public void refreshDepend(boolean dirty, boolean load)
			throws RemoteException {
		this.param.refreshDependency();
	}

	@Override
	public void refreshDependency() throws RemoteException {
		// TODO Auto-generated method stub

	}

	@Override
	public CParamMaterial getClientImpl() {
		return this;
	}
}
