/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_4.
 * 
 * - Rename file-date node to MD5 <br>
 * - Rename GeoData node to Data <br>
 * - Rename item node of depend to entry <br>
 * - Remove temporary node in structure
 * 
 * @author Werner Koppelsttter
 */
class Converter_4 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {

		Converter_4.updateXMLStructure(node);
		return node;
	}

	/**
	 * Recursive update of structure
	 * 
	 * @param node
	 *            Data to convert
	 */
	private static void updateXMLStructure(XMLNode node) {
		XMLNode tmp;

		if (node.getName().compareTo("Depend") == 0) {
			Converter_4.updateDepend(node);
		} else if (node.getName().compareTo("GeoData") == 0) {
			node.setName("Data");
		} else if (node.getName().compareTo("DateNum") == 0) {
			node.setName("MD5");
		}

		// next Node
		int i = 0;
		while (i < node.getNodeSize()) {
			tmp = node.getNode(i);
			if (tmp.getName().compareTo("Temporary") == 0) {
				node.rmNode(i);
			} else {
				Converter_4.updateXMLStructure(tmp);
				i++;
			}
		}
	}

	/**
	 * Recursive update of dependencies
	 * 
	 * @param node
	 *            Data to convert
	 */
	private static void updateDepend(XMLNode node) {
		if (node.getName().compareTo("item") == 0) {
			node.setName("entry");
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_4.updateDepend(n);
		}
	}

}
