/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.ipython;

import java.io.ByteArrayOutputStream;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import org.json.simple.JSONObject;
import org.zeromq.ZMQ;

import at.lcm.bu21.general.dtypes.TData;

/**
 * Send data to IPython
 * 
 * @author Gnther Weidenholzer
 * @author Werner Koppelsttter
 */
public class SendToIPython {

	/** ZMQ context */
	private static ZMQ.Context context = ZMQ.context(1);

	/**
	 * Sent data to IPyhton
	 * 
	 * @param name
	 *            Name of variable
	 * @param data
	 *            Data to send
	 * @param cf
	 *            Connection file
	 * @throws Exception
	 */
	public static void sendToIPython(String name, TData data, ConnectionFile cf) throws Exception {

		if (cf == null) {
			cf = ConnectionFile.findConnectionFiles().get(0);
		}

		// connect to kernel
		KernelConnection kernel = new KernelConnection(cf, SendToIPython.context);

		// import pacakges
		SendToIPython.execute_request(kernel, "import dtypes.xml");
		SendToIPython.execute_request(kernel, "from dtypes.xml import fromString");

		// prepare code string
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		XMLStreamWriter xmlWriter = XMLOutputFactory.newInstance().createXMLStreamWriter(bos, "UTF-8");

		data.obj2xml(xmlWriter);
		xmlWriter.close();
		bos.close();

		String xmlstr = bos.toString("UTF-8");
		String code = name + "=fromString('" + xmlstr + "')";

		SendToIPython.execute_request(kernel, code);

	}

	/**
	 * Send command
	 * 
	 * @param kernel
	 * @param code
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	private static void execute_request(KernelConnection kernel, String code) throws Exception {
		String msg_type = "execute_request";

		JSONObject content = new JSONObject();
		content.put("code", code);
		content.put("silent", false);
		content.put("store_history", true);
		JSONObject user_expressions = new JSONObject();

		// user_expressions.put("result", "a+b");
		content.put("user_expressions", user_expressions);
		content.put("allow_stdin", true);

		MessageHeader parent = null;

		Message msg = new Message(kernel, msg_type, content, parent, null, null, null);

		kernel.send(kernel.shell, msg);
	}

}
