/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui;

import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import magopt.gui.MagOptGui.ParamType;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;

/**
 * Creates Menu Bar for MagOpt UI.
 * 
 * @author Aigner Markus
 * 
 */
public class MenuBar extends JMenuBar {
	private static final long serialVersionUID = 1L;

	private MagOptGui mGui;

	private JMenu mEdit;
	private JMenuItem mSave, mSaveAs;
	private JMenuItem mInsert, mCut, mCopy, mPaste, mDelete;
	private JMenuItem mRefresh, mCheck, mRegenerateDs, mRegenerateAll;
	private JMenuItem mDuplicateDs, mReduceDs;

	protected MenuBar(MagOptGui gui) {
		this.mGui = gui;

		/* file menu */
		JMenu menuFile = new JMenu("File");

		/* new project */
		JMenuItem miNew = new JMenuItem("New");
		miNew.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.newProject();
			}
		});
		menuFile.add(miNew);

		/* open project */
		JMenuItem miOpen = new JMenuItem("Open");
		miOpen.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.openProject();
			}
		});
		menuFile.add(miOpen);

		/* save project as */
		menuFile.addSeparator();
		this.mSaveAs = new JMenuItem("Save As");
		this.mSaveAs.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.saveProject(false);
			}
		});
		menuFile.add(this.mSaveAs);

		/* save project */
		this.mSave = new JMenuItem("Save");
		this.mSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				Event.CTRL_MASK));
		this.mSave.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.saveProject(true);
			}
		});
		menuFile.add(this.mSave);

		/* exit */
		menuFile.addSeparator();
		JMenuItem miExit = new JMenuItem("Exit");
		miExit.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.quitGui();
			}
		});
		menuFile.add(miExit);
		this.add(menuFile);

		/* edit menu */
		this.mEdit = new JMenu("Edit");

		/* insert (sub-)menu */
		this.mInsert = new JInsertMenu();
		this.mEdit.add(this.mInsert);

		/* cut param */
		this.mEdit.addSeparator();
		this.mCut = new JMenuItem("Cut");
		// this.mCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
		// Event.CTRL_MASK));
		this.mCut.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.copyParameter(true);
			}
		});
		this.mEdit.add(this.mCut);

		/* copy param */
		this.mCopy = new JMenuItem("Copy");
		// this.mCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
		// Event.CTRL_MASK));
		this.mCopy.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.copyParameter(false);
			}
		});
		this.mEdit.add(this.mCopy);

		/* paste param */
		this.mPaste = new JMenuItem("Paste");
		// this.mPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
		// Event.CTRL_MASK));
		this.mPaste.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.pasteParameter();
			}
		});
		this.mEdit.add(this.mPaste);

		/* delete param */
		this.mEdit.addSeparator();
		this.mDelete = new JMenuItem("Delete");
		this.mDelete.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.deleteComponent();
			}
		});
		this.mDelete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,
				0));
		this.mEdit.add(this.mDelete);

		/* add config */
		this.mEdit.addSeparator();
		this.mDuplicateDs = new JMenuItem("Duplicate Data Set");
		this.mDuplicateDs.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				MenuBar.this.mGui.addDataSet();
			}
		});
		this.mEdit.add(this.mDuplicateDs);

		/* remove config */
		this.mReduceDs = new JMenuItem("Remove Data Set");
		this.mReduceDs.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				MenuBar.this.mGui.removeDataSet();
			}
		});
		this.mEdit.add(this.mReduceDs);

		this.add(this.mEdit);

		/* update menu */
		JMenu menuUpdate = new JMenu("Update");

		/* refresh */
		this.mRefresh = new JMenuItem("Refresh");
		this.mRefresh.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.refresh();
			}
		});
		menuUpdate.add(this.mRefresh);

		/* check */
		menuUpdate.addSeparator();
		this.mCheck = new JMenuItem("Check");
		this.mCheck.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.check();
			}
		});
		menuUpdate.add(this.mCheck);

		/* regenerate ds */
		menuUpdate.addSeparator();
		this.mRegenerateDs = new JMenuItem("Regenerate Current Data Set");
		this.mRegenerateDs.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.regenerate(false);
			}
		});
		menuUpdate.add(this.mRegenerateDs);

		/* regenerate ds */
		this.mRegenerateAll = new JMenuItem("Regenerate All Data Sets");
		this.mRegenerateAll.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				MenuBar.this.mGui.regenerate(true);
			}
		});
		menuUpdate.add(this.mRegenerateAll);

		this.add(menuUpdate);

		this.check();
	}

	/**
	 * Enable/Disable Menu Bar Elements.
	 */
	protected void check() {
		if (this.mGui.mProject == null) {
			this.mSave.setEnabled(false);
			this.mSaveAs.setEnabled(false);
			this.mDelete.setEnabled(false);
			this.mInsert.setEnabled(false);
			this.mRefresh.setEnabled(false);
			this.mCheck.setEnabled(false);
			this.mRegenerateDs.setEnabled(false);
			this.mRegenerateAll.setEnabled(false);
			this.mDuplicateDs.setEnabled(false);
			this.mReduceDs.setEnabled(false);
			this.mCut.setEnabled(false);
			this.mCopy.setEnabled(false);
			this.mPaste.setEnabled(false);
		} else {
			this.mSave.setEnabled(true);
			this.mSaveAs.setEnabled(true);
			this.mInsert.setEnabled(true);

			/*
			 * disables menu items 'delete' and 'copy' if selected item is
			 * project
			 */
			IParameter selected = this.mGui.mProjectTree.getSelectedParameter();
			if (selected instanceof IProject) {
				this.mDelete.setEnabled(false);
				this.mCopy.setEnabled(false);
				this.mCut.setEnabled(false);
			} else {
				this.mDelete.setEnabled(true);
				this.mCopy.setEnabled(true);
				this.mCut.setEnabled(true);
			}
			this.mRefresh.setEnabled(true);
			this.mCheck.setEnabled(true);
			this.mRegenerateDs.setEnabled(true);
			this.mRegenerateAll.setEnabled(true);
			this.mDuplicateDs.setEnabled(true);
			this.mReduceDs.setEnabled(true);
			this.mPaste.setEnabled(true);
		}
	}

	/**
	 * Creates Insert Menu.
	 * 
	 * @author Aigner Markus
	 * 
	 */
	class JInsertMenu extends JMenu {
		private static final long serialVersionUID = 1L;

		protected JInsertMenu() {
			this.setText("Insert");

			/* add new container */
			JMenuItem miContainer = new JMenuItem("Container");
			miContainer.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.CONTAINER);
				}
			});
			this.add(miContainer);

			/* add new paramfield */
			JMenuItem miField = new JMenuItem("Field");
			miField.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.FIELD);
				}
			});
			this.add(miField);

			/* add new param function */
			JMenuItem miFunction = new JMenuItem("Function");
			miFunction.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.FUNCTION);
				}
			});
			this.add(miFunction);

			/* add new param formula */
			JMenuItem miFormula = new JMenuItem("Formula");
			miFormula.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.FORMULA);
				}
			});
			this.add(miFormula);

			/* add new param directory */
			JMenuItem miDirectory = new JMenuItem("Directory");
			miDirectory.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.DIRECTORY);
				}
			});
			this.add(miDirectory);

			/* add new param file */
			JMenuItem miFile = new JMenuItem("File");
			miFile.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.FILE);
				}
			});
			this.add(miFile);

			this.addSeparator();

			/* add new template */
			JMenuItem miTemplate = new JMenuItem("Template");
			miTemplate.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					MenuBar.this.mGui.insertComponent(ParamType.TEMPLATE);
				}
			});
			this.add(miTemplate);
		}
	}
}
