/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.compare;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.magopt.structure.compare.CompareResult;

/**
 * Table model for compare-results
 * 
 * @author Werner Koppelsttter
 */
public class CompareModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** List of compare-results */
	protected CompareResult[] results = null;

	/**
	 * Set compare-results
	 * 
	 * @param results
	 *            List of compare-results
	 */
	public void setCompareResult(CompareResult[] results) {
		this.results = results;

		// refresh table of model
		for (TableModelListener listener : this
				.getListeners(TableModelListener.class)) {
			if ((listener instanceof JTable)
					&& this.equals(((JTable) listener).getModel())) {
				listener.tableChanged(new TableModelEvent(this));
			}
		}
	}

	@Override
	public int getColumnCount() {
		return CompareColumn.values().length;
	}

	@Override
	public int getRowCount() {
		int len = 0;

		if (this.results != null) {
			len = this.results.length;
		}

		return len;
	}

	@Override
	public Object getValueAt(int row, int column) {
		Object obj = null;
		CompareResult res;

		if (this.results != null) {
			res = this.results[row];
			if (CompareColumn.IDENTIFIER.getValue() == column) {
				obj = res.getIdentifier();
			} else if (CompareColumn.REASON.getValue() == column) {
				obj = res.getReason();
			} else if (CompareColumn.DIFFERENT.getValue() == column) {
				obj = res.getDifference();
			} else {
				obj = res;
			}
		}

		return obj;
	}

	@Override
	public String getColumnName(int column) {
		for (CompareColumn e : CompareColumn.values()) {
			if (column == e.getValue()) {
				return e.getName();
			}
		}
		return null;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		return String.class;
	}

}
