/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.event;

import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.magopt.structure.event.Event;
import at.lcm.bu21.magopt.structure.event.EventLog;

/**
 * Event-model for table
 * 
 * @author Werner Koppelsttter
 */
public class EventModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** List of events */
	protected EventLog events = null;

	/** Display info message */
	protected boolean info = true;
	/** Display warning message */
	protected boolean warning = true;
	/** Display error message */
	protected boolean error = true;

	/**
	 * Set event-list
	 * 
	 * @param events
	 *            Event list
	 * @param info
	 *            Display infos
	 * @param warning
	 *            Display warnings
	 * @param error
	 *            Display errors
	 */
	public void setEvents(EventLog events, boolean info, boolean warning,
			boolean error) {
		this.events = events;
		this.info = info;
		this.warning = warning;
		this.error = error;
		this.fireTableDataChanged();
	}

	@Override
	public int getColumnCount() {
		return EventColumn.values().length;
	}

	@Override
	public int getRowCount() {
		int len = 0;

		if (this.events != null) {
			len = this.events.getBuffer(this.info, this.warning, this.error).length;
		}

		return len;
	}

	@Override
	public Object getValueAt(int row, int column) {
		Object obj = null;

		if (this.events != null) {
			obj = this.events.getBuffer(this.info, this.warning, this.error)[row];
		}

		return obj;
	}

	@Override
	public String getColumnName(int column) {
		for (EventColumn e : EventColumn.values()) {
			if (column == e.getValue()) {
				return e.getName();
			}
		}
		return null;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		return Event.class;
	}

}
