/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.helpers;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * PreConfigured FileChooser for loading/saving/creating MagOpt Projects and
 * Templates.
 * 
 * @author Aigner Markus
 * 
 */
public class JProjectChooser extends JFileChooser {
	private static final long serialVersionUID = 1L;

	public final static int ALL = 0;
	public final static int PROJECT_ONLY = 1;
	public final static int TEMPLATE_ONLY = 2;
	public final static int COMPONENT_ONLY = 3;

	public JProjectChooser(int mode) {
		super();

		FileNameExtensionFilter fProject = new FileNameExtensionFilter("Project (*.mop)", "mop");
		FileNameExtensionFilter fTemplate = new FileNameExtensionFilter("Template (*.mopt)", "mopt");
		FileNameExtensionFilter fComponent = new FileNameExtensionFilter("Component (*.mopc)", "mopc");

		switch (mode) {
		case ALL:
			this.addChoosableFileFilter(fProject);
			this.addChoosableFileFilter(fTemplate);
			this.addChoosableFileFilter(fComponent);
			this.setFileFilter(fProject);
			break;
		case PROJECT_ONLY:
			this.addChoosableFileFilter(fProject);
			this.setFileFilter(fProject);
			break;
		case TEMPLATE_ONLY:
			this.addChoosableFileFilter(fTemplate);
			this.setFileFilter(fTemplate);
			break;
		case COMPONENT_ONLY:
			this.addChoosableFileFilter(fComponent);
			this.setFileFilter(fComponent);
			break;
		default:
			throw new IllegalArgumentException("Invalid Filter Mode");
		}

		this.setAcceptAllFileFilterUsed(false);
		this.setDialogTitle("Choose File");
		this.setFileSelectionMode(JFileChooser.FILES_ONLY);
		this.setMultiSelectionEnabled(false);
		this.setFileHidingEnabled(true);
	}

	public JProjectChooser() {
		this(JProjectChooser.ALL);
	}

	public JProjectChooser(int mode, File directory) {
		this(mode);
		this.setCurrentDirectory(directory);
	}

	@Override
	public File getSelectedFile() {
		File file = super.getSelectedFile();
		if (file == null) {
			return (file);
		}
		String ext = ((FileNameExtensionFilter) this.getFileFilter()).getExtensions()[0];
		String fName = super.getSelectedFile().getAbsolutePath();

		if (fName.endsWith("." + ext) == false) {
			fName = fName + "." + ext;
		}
		return (new File(fName));
	}
}
