/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.icons;

import javax.swing.ImageIcon;

import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParamDirectory;
import at.lcm.bu21.magopt.structure.IParamField;
import at.lcm.bu21.magopt.structure.IParamFile;
import at.lcm.bu21.magopt.structure.IParamFormula;
import at.lcm.bu21.magopt.structure.IParamJavaFunction;
import at.lcm.bu21.magopt.structure.IParamLinkedField;
import at.lcm.bu21.magopt.structure.IParamMaterial;
import at.lcm.bu21.magopt.structure.IParamMatlabFunction;
import at.lcm.bu21.magopt.structure.IParamPythonFunction;
import at.lcm.bu21.magopt.structure.IParamPythonFunction_old;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;

/**
 * Parameter Icons
 * 
 * @author Werner Koppelsttter
 */
public class IconParameter {

	/** Icon for parameter Project */
	public final static ImageIcon ICON_PROJECT = new ImageIcon(
			IconParameter.class.getResource("/magopt/gui/icons/Project.png"));
	/** Icon for parameter Container */
	public final static ImageIcon ICON_CONTAINER = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamContainer.png"));
	/** Icon for parameter JavaFunction */
	public final static ImageIcon ICON_JAVA = new ImageIcon(IconParameter.class
			.getResource("/magopt/gui/icons/ParamJavaFunction.png"));
	/** Icon for parameter MatlabFunction */
	public final static ImageIcon ICON_MATLAB = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamMatlabFunction.png"));
	/** Icon for parameter MatlabFunction */
	public final static ImageIcon ICON_PYTHON = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamPythonFunction.png"));
	/** Icon for parameter Material */
	public final static ImageIcon ICON_MATERIAL = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamMaterial.png"));
	/** Icon for parameter Field */
	public final static ImageIcon ICON_FIELD = new ImageIcon(IconParameter.class
			.getResource("/magopt/gui/icons/ParamField.png"));
	/** Icon for parameter Formula */
	public final static ImageIcon ICON_FORMULA = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamFormula.png"));
	/** Icon for parameter LinkedField */
	public final static ImageIcon ICON_LINKEDFIELD = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamLinkedField.png"));
	/** Icon for parameter File */
	public final static ImageIcon ICON_FILE = new ImageIcon(
			IconParameter.class.getResource("/magopt/gui/icons/ParamFile.png"));
	/** Icon for parameter Directory */
	public final static ImageIcon ICON_DIRECTORY = new ImageIcon(
			IconParameter.class
					.getResource("/magopt/gui/icons/ParamDirectory.png"));

	/**
	 * Get icon for selected parameter
	 * 
	 * @param param
	 *            Parameter
	 * @return Icon for parameter
	 */
	public static ImageIcon getIcon(IParameter param) {
		ImageIcon icon = null;

		if (param instanceof IProject) {
			icon = IconParameter.ICON_PROJECT;
		} else if (param instanceof IParamFormula) {
			icon = IconParameter.ICON_FORMULA;
		} else if (param instanceof IParamLinkedField) {
			icon = IconParameter.ICON_LINKEDFIELD;
		} else if (param instanceof IParamField) {
			icon = IconParameter.ICON_FIELD;
		} else if (param instanceof IParamMatlabFunction) {
			icon = IconParameter.ICON_MATLAB;
		} else if (param instanceof IParamPythonFunction) {
			icon = IconParameter.ICON_PYTHON;
		} else if (param instanceof IParamPythonFunction_old) {
			icon = IconParameter.ICON_PYTHON;
		} else if (param instanceof IParamJavaFunction) {
			icon = IconParameter.ICON_JAVA;
		} else if (param instanceof IParamMaterial) {
			icon = IconParameter.ICON_MATERIAL;
		} else if (param instanceof IParamContainer) {
			icon = IconParameter.ICON_CONTAINER;
		} else if (param instanceof IParamFile) {
			icon = IconParameter.ICON_FILE;
		} else if (param instanceof IParamDirectory) {
			icon = IconParameter.ICON_DIRECTORY;
		}

		return icon;
	}
}
