/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.component;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;

import magopt.gui.view.ParamBaseNoScroll;
import at.lcm.bu21.magopt.structure.IComponent;

/**
 * Display linked files of component
 * 
 * @author Werner Koppelsttter
 */
public class LinkedFilesView extends ParamBaseNoScroll {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** ParamComponent to display */
	private IComponent _component;

	/** Table to display linked files */
	private JTable table;

	/** Files-model */
	private LinkedFilesModel model;

	// /** Button to add files */
	// private JButton addFile;

	// /** Button to remove files */
	// private JButton removeFile;

	/** Label to display component-folder */
	private JTextField fieldFolder;

	/** Button to open component-folder */
	private JButton openFolder;

	// /** Label to display help-path */
	// private JTextField fieldHelp;

	// /** Button to add help-file */
	// private JButton addHelp;

	/**
	 * Create container-summary view
	 */
	public LinkedFilesView() {
		JPanel panelFiles, panelFolder;
		// JPanel panelButton;
		// JPanel panelHelp;
		JScrollPane scroll;

		this.setLayout(new BorderLayout());
		this.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		// component folder
		panelFolder = new JPanel();
		panelFolder.setLayout(new BorderLayout());
		panelFolder.setBorder(new EmptyBorder(5, 5, 5, 5));
		panelFolder.add(new JLabel("Component-folder"), BorderLayout.WEST);

		this.fieldFolder = new JTextField();
		this.fieldFolder.setEnabled(false);
		panelFolder.add(this.fieldFolder, BorderLayout.CENTER);

		this.openFolder = new JButton("Open");
		this.openFolder.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					Desktop.getDesktop().open(
							LinkedFilesView.this._component.getFolder());
				} catch (IOException e1) {
				}
			}
		});
		panelFolder.add(this.openFolder, BorderLayout.EAST);

		this.add(panelFolder, BorderLayout.NORTH);

		// linked files
		panelFiles = new JPanel();
		panelFiles.setLayout(new BorderLayout());
		panelFiles.setBorder(new EmptyBorder(5, 5, 5, 5));
		panelFiles.add(new JLabel("Linked-files"), BorderLayout.NORTH);

		this.model = new LinkedFilesModel();
		this.model.addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				LinkedFilesView.this.fireParamChanged();
			}
		});

		this.table = new JTable();
		this.table.setModel(this.model);
		this.table.getTableHeader().setReorderingAllowed(false);
		// this.table.setColumnSelectionAllowed(true);
		this.table.setRowSelectionAllowed(true);
		this.table
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		this.table.setDefaultRenderer(Object.class,
				new DefaultTableCellRenderer() {

					/** Serialization version number */
					private static final long serialVersionUID = 1L;

					@Override
					public java.awt.Component getTableCellRendererComponent(
							JTable table, Object value, boolean isSelected,
							boolean hasFocus, int row, int column) {
						super.getTableCellRendererComponent(table, value,
								isSelected, hasFocus, row, column);

						this.setToolTipText(((File) LinkedFilesView.this.model
								.getValueAt(row, -1)).getPath());

						return this;
					}
				});
		// this.table.getSelectionModel().addListSelectionListener(
		// new ListSelectionListener() {
		// @Override
		// public void valueChanged(ListSelectionEvent e) {
		// LinkedFilesView.this.removeFile
		// .setEnabled(LinkedFilesView.this.table
		// .getSelectedRowCount() != 0);
		// }
		// });

		// define size
		this.table.getColumnModel()
				.getColumn(LinkedFilesColumn.EXIST.getValue())
				.setPreferredWidth(70);

		this.table.getColumnModel()
				.getColumn(LinkedFilesColumn.EXIST.getValue()).setMaxWidth(40);

		scroll = new JScrollPane();
		scroll.setViewportView(this.table);
		panelFiles.add(scroll, BorderLayout.CENTER);

		// panelButton = new JPanel();
		// panelButton.setLayout(new GridLayout(1, 2));
		// panelButton.setBorder(new EmptyBorder(5, 5, 5, 5));
		//
		// this.addFile = new JButton("Link File");
		// this.addFile.addActionListener(new ActionListener() {
		// @Override
		// public void actionPerformed(ActionEvent e) {
		// JFileChooser chooser;
		//
		// chooser = new JFileChooser(
		// ((Project) LinkedFilesView.this._component
		// .getContainer().getRoot())
		// .getAllFilesDirectory());
		// chooser.setDialogTitle("Select linked-files");
		// chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		// chooser.setMultiSelectionEnabled(true);
		// if (chooser.showOpenDialog(LinkedFilesView.this) ==
		// JFileChooser.APPROVE_OPTION) {
		// for (File f : chooser.getSelectedFiles()) {
		// LinkedFilesView.this._component.addLinkedFile(f);
		// }
		// LinkedFilesView.this.fireParamChanged();
		// }
		// }
		// });
		//
		// this.removeFile = new JButton("Remove File");
		// this.removeFile.setEnabled(false);
		// this.removeFile.addActionListener(new ActionListener() {
		// @Override
		// public void actionPerformed(ActionEvent event) {
		// int[] indices;
		// File[] files;
		//
		// indices = LinkedFilesView.this.table.getSelectedRows();
		// files = new File[indices.length];
		// for (int k = 0; k < files.length; k++) {
		// files[k] = (File) LinkedFilesView.this.model.getValueAt(
		// indices[k], -1);
		// }
		//
		// for (File f : files) {
		// LinkedFilesView.this._component.removeLinkedFile(f);
		// }
		// LinkedFilesView.this.fireParamChanged();
		// }
		// });
		//
		// panelButton.add(this.addFile);
		// panelButton.add(this.removeFile);
		//
		// panelFiles.add(panelButton, BorderLayout.SOUTH);

		this.add(panelFiles, BorderLayout.CENTER);

	}

	/**
	 * Sets new ParamComponent.
	 * 
	 * @param newComponent
	 *            - ParamComponent
	 */
	public void setComponent(IComponent newComponent) {
		this._component = newComponent;
		this.update();
	}

	@Override
	protected void update() {
		File file;
		String path = "";

		this.model.setComponent(this._component);
		if (this._component != null) {
			try {
				file = this._component.getFolder();
				if (file != null) {
					path = file.getPath();
				}
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
		this.fieldFolder.setText(path);
	}

}
