/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.algorithm;

import javax.swing.JPanel;

/**
 * Interface to enable/disable solver settings
 * 
 * @author Werner Koppelsttter
 */
public abstract class JPanelSolver extends JPanel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Panel enabled */
	protected boolean enable = true;

	/**
	 * Are settings enabled
	 * 
	 * @return Setting enable
	 */
	public boolean isSettingEnabled() {
		return this.enable;
	}

	/**
	 * Set settings enabled
	 * 
	 * @param enable
	 *            Settings enable
	 */
	public void setSettingEnabled(boolean enable) {
		this.enable = enable;
		this.refreshEnabled();
	}

	/**
	 * Refresh enable state of components
	 */
	public abstract void refreshEnabled();
}
