/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.gui.mapping;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.LineBorder;

import at.lcm.bu21.optim.core.Optimization;
import at.lcm.bu21.optim.core.mapping.MappingManager;
import at.lcm.bu21.optim.core.mapping.utils.dstructures.TMappingConstants.TThreadState;

/**
 * Display panel for mapping management
 * 
 * @author Ciprian Zavoianu
 */
public class PanelMapping extends JPanel {

	class MappingCreationButtonActionLister implements ActionListener {

		private final String targetName;
		private final String actionToPerform;

		private MappingCreationButtonActionLister(String targetName, String actionToPerform) {
			this.targetName = targetName;
			this.actionToPerform = actionToPerform;
		}

		@Override
		public void actionPerformed(ActionEvent event) {
			if (this.actionToPerform.equals("RUN")) {
				PanelMapping.this.parentMappingManager.setFastTrainingEnabled(this.targetName,
						PanelMapping.this.creationFTCheckBoxes.get(this.targetName).isSelected());
				for (String name : PanelMapping.this.parentMappingManager.getTargetNames()) {
					PanelMapping.this.parentMappingManager.setReducedTrainingSetPreference(name,
							PanelMapping.this.creationSizeTextFields.get(name).getText());
					PanelMapping.this.parentMappingManager.setMaxTrainingDurationPreference(name,
							PanelMapping.this.creationDurationTextFields.get(name).getText());
				}
				PanelMapping.this.parentMappingManager.runIndividualMappingCreation(this.targetName);
			}
			if (this.actionToPerform.equals("RETRAIN")) {
				for (String name : PanelMapping.this.parentMappingManager.getTargetNames()) {
					PanelMapping.this.parentMappingManager.setReducedTrainingSetPreference(name,
							PanelMapping.this.creationSizeTextFields.get(name).getText());
					PanelMapping.this.parentMappingManager.setMaxTrainingDurationPreference(name,
							PanelMapping.this.creationDurationTextFields.get(name).getText());

				}
				PanelMapping.this.parentMappingManager.runIndividualMappingRetraining(this.targetName);

			}
			if (this.actionToPerform.equals("STOP")) {
				PanelMapping.this.parentMappingManager.stopIndividualMappingCreation(this.targetName);
			}
		}
	}

	class FTActionLister implements ActionListener {

		private final String targetName;

		private FTActionLister(String targetName) {
			this.targetName = targetName;
		}

		@Override
		public void actionPerformed(ActionEvent event) {
			PanelMapping.this.parentMappingManager.setFastTrainingEnabled(this.targetName,
					PanelMapping.this.creationFTCheckBoxes.get(this.targetName).isSelected());
		}
	}

	class UECActionLister implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent event) {
			PanelMapping.this.parentMappingManager
					.setUseErrorsClassifier(PanelMapping.this.optimUECCheckBox.isSelected());
		}
	}

	class OptimizationControlButtonActionLister implements ActionListener {

		private final String actionToPerform;

		private OptimizationControlButtonActionLister(String actionToPerform) {
			this.actionToPerform = actionToPerform;
		}

		@Override
		public void actionPerformed(ActionEvent event) {

			if (this.actionToPerform.equals("RUN")) {
				PanelMapping.this.parentMappingManager.runMappingBasedOptimization();
			}
			if (this.actionToPerform.equals("STOP")) {
				PanelMapping.this.parentMappingManager.stopMappingBasedOptimization();
			}
			if (this.actionToPerform.equals("DISCARD")) {
				PanelMapping.this.parentMappingManager.clearMappingMemory();
			}
			if (this.actionToPerform.equals("VERIFY")) {
				PanelMapping.this.parentMappingManager.verifyBestIndividuals(
						(Integer) PanelMapping.this.optimSolutionsToReevaluateSpinner.getValue());
			}

		}
	}

	private static final long serialVersionUID = -5054958560350093972L;

	private JPanel creationPanel;
	private JPanel optimizationPanel;

	private Map<String, JLabel> nameLabels = new HashMap<String, JLabel>();
	private Map<String, JLabel> qualityLabels = new HashMap<String, JLabel>();
	private Map<String, JLabel> sizeLabels = new HashMap<String, JLabel>();
	private List<JLabel> creationHeaderLabels = new ArrayList<JLabel>();
	private List<JSeparator> separators = new ArrayList<JSeparator>();
	private Map<String, JButton> creationRunButtons = new HashMap<String, JButton>();
	private Map<String, JButton> creationRetrainButtons = new HashMap<String, JButton>();
	private Map<String, JButton> creationStopButtons = new HashMap<String, JButton>();
	private Map<String, JButton> optimizationButtons = new HashMap<String, JButton>();
	private Map<String, JCheckBox> creationFTCheckBoxes = new HashMap<String, JCheckBox>();
	private Map<String, JTextField> creationDurationTextFields = new HashMap<String, JTextField>();
	private Map<String, JTextField> creationSizeTextFields = new HashMap<String, JTextField>();
	private Map<String, JLabel> creationDurationLabels = new HashMap<String, JLabel>();
	private Map<String, JLabel> creationSizeLabels = new HashMap<String, JLabel>();
	private JProgressBar optimProgressBar;
	private JLabel optimInfoLabel;
	private JSpinner optimSolutionsToReevaluateSpinner;
	private JCheckBox optimUECCheckBox;
	// private JSpinner optimTrainingSetSize;
	// private JSpinner optimTrainingSetDuration;
	// private JLabel optimDurationLabel;
	// private JLabel optimSizeLabel;

	private MappingManager parentMappingManager;
	private JTextField textField;

	/**
	 * Create panel. Used by WindowsBuilder
	 * 
	 * @throws Exception
	 */
	private PanelMapping() throws Exception {

		this.textField = new JTextField();
		this.textField.setText("5000000001");
		this.add(this.textField);
		this.textField.setColumns(10);

		JCheckBox chckbxNewCheckBox = new JCheckBox("FT");
		chckbxNewCheckBox.setSelected(true);
		this.add(chckbxNewCheckBox);

	}

	public PanelMapping(Optimization optim) {
		this.parentMappingManager = optim.getMappingControl();
		this.setLayout(null);

		this.generateUIComponents();
		this.refresh();
	}

	public void generateUIComponents() {

		int labelWidth = 100;
		int labelHeight = 20;
		int smallLabelWidth = 40;
		int textFieldWidth = 40;
		int labelYSpacing = 2 * labelHeight;
		int labelXSpacing = labelHeight;
		int smallLabelSpacing = 5;
		int labelStartY = 10;
		int buttonWidth = 250;
		int buttonHeight = 30;

		int statusLabelWidth = 500;

		int nameLabelStartX = 10;
		int qualityLabelStartX = nameLabelStartX + labelWidth + labelXSpacing;
		int sizeLabelStartX = qualityLabelStartX + labelWidth + labelXSpacing;
		int optionsLabelStartX = sizeLabelStartX + labelWidth + labelXSpacing;
		int buttonStartX = optionsLabelStartX + (2 * labelWidth) + labelXSpacing;

		int panelStartX = 20;
		int panelWidth = buttonStartX + buttonWidth + 10;

		int optimizationPanelStartY = 20;
		int optimizationPanelHeight = 40;
		int creationPanelStartY = optimizationPanelStartY + optimizationPanelHeight + 20;
		int creationPanelHeight = 0;

		// optimization panel components

		// status label
		this.optimInfoLabel = new JLabel("0 mapping-based designs ready for verification!");
		this.optimInfoLabel.setBounds(nameLabelStartX, labelStartY, statusLabelWidth, labelHeight);
		this.optimInfoLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		// the progress bar
		this.optimProgressBar = new JProgressBar();
		this.optimProgressBar.setStringPainted(true);
		this.optimProgressBar.setValue(0);
		this.optimProgressBar.setBounds(nameLabelStartX, labelStartY, statusLabelWidth, labelHeight);

		// the spinner
		this.optimSolutionsToReevaluateSpinner = new JSpinner();
		this.optimSolutionsToReevaluateSpinner.setModel(
				new SpinnerNumberModel(100, 1, this.parentMappingManager.getProblem().getNumObjectives() * 100, 50));
		this.optimSolutionsToReevaluateSpinner.setBounds(buttonStartX + (int) ((3.25 * buttonWidth) / 8),
				(labelStartY - ((labelYSpacing - buttonHeight) / 2)) + 3, (int) ((1.5 * buttonWidth) / 8),
				buttonHeight - 6);

		// the use errors classifier check box
		this.optimUECCheckBox = new JCheckBox("use Errors-Classifier");
		this.optimUECCheckBox.setBounds(buttonStartX - ((buttonWidth / 2) + labelXSpacing),
				labelStartY - ((labelYSpacing - buttonHeight) / 2), buttonWidth / 2, buttonHeight);
		this.optimUECCheckBox.setSelected(false);
		this.optimUECCheckBox.addActionListener(new UECActionLister());

		// the control buttons
		// RUN
		JButton button = new JButton("Run new optimization");
		button.setBounds(buttonStartX, labelStartY - ((labelYSpacing - buttonHeight) / 2), buttonWidth, buttonHeight);
		button.addActionListener(new OptimizationControlButtonActionLister("RUN"));
		this.optimizationButtons.put("RUN", button);

		// STOP
		button = new JButton("Stop ");
		button.setBounds(buttonStartX, labelStartY - ((labelYSpacing - buttonHeight) / 2), buttonWidth, buttonHeight);
		button.addActionListener(new OptimizationControlButtonActionLister("STOP"));
		this.optimizationButtons.put("STOP", button);

		// VERIFY
		button = new JButton("Verify best");
		button.setBounds(buttonStartX, labelStartY - ((labelYSpacing - buttonHeight) / 2), (3 * buttonWidth) / 8,
				buttonHeight);
		button.addActionListener(new OptimizationControlButtonActionLister("VERIFY"));
		this.optimizationButtons.put("VERIFY", button);

		// DISCARD
		button = new JButton("Discard");
		button.setBounds(buttonStartX + ((5 * buttonWidth) / 8), labelStartY - ((labelYSpacing - buttonHeight) / 2),
				(3 * buttonWidth) / 8, buttonHeight);
		button.addActionListener(new OptimizationControlButtonActionLister("DISCARD"));
		this.optimizationButtons.put("DISCARD", button);

		// creation panel components
		// the header labels
		JLabel label = new JLabel("Target name");
		label.setBounds(nameLabelStartX, labelStartY, labelWidth, labelHeight);
		label.setFont(new Font("Tahoma", Font.BOLD, 11));
		this.creationHeaderLabels.add(label);

		label = new JLabel("Estim. accuracy");
		label.setBounds(qualityLabelStartX, labelStartY, labelWidth, labelHeight);
		label.setFont(new Font("Tahoma", Font.BOLD, 11));
		this.creationHeaderLabels.add(label);

		label = new JLabel("Actual TS #");
		label.setBounds(sizeLabelStartX, labelStartY, labelWidth, labelHeight);
		label.setFont(new Font("Tahoma", Font.BOLD, 11));
		this.creationHeaderLabels.add(label);

		label = new JLabel("Current training options");
		label.setBounds(optionsLabelStartX, labelStartY, 2 * labelWidth, labelHeight);
		label.setFont(new Font("Tahoma", Font.BOLD, 11));
		this.creationHeaderLabels.add(label);

		// the separators
		for (int i = 0; i < this.parentMappingManager.getNumMappingTargets(); i++) {
			JSeparator separator = new JSeparator();
			separator.setBounds(nameLabelStartX, (((i + 1) * labelYSpacing) / 2) + ((i + 1) * labelHeight),
					(buttonStartX + buttonWidth) - 10, 5);
			this.separators.add(separator);
		}

		int count = 1;
		for (String targetName : this.parentMappingManager.getTargetNames()) {
			// the target name labels
			String value = targetName.substring(targetName.lastIndexOf(".") + 1);
			label = new JLabel(value);
			label.setBounds(nameLabelStartX, labelStartY + (count * labelYSpacing), labelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.nameLabels.put(targetName, label);

			// the mapping quality labels
			value = this.parentMappingManager.getMappingQuality(targetName);
			label = new JLabel(value);
			label.setBounds(qualityLabelStartX, labelStartY + (count * labelYSpacing), labelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.qualityLabels.put(targetName, label);

			// the actual number of samples labels
			value = this.parentMappingManager.getNrOfTrainingSamples(targetName);
			label = new JLabel(value);
			label.setBounds(sizeLabelStartX, labelStartY + (count * labelYSpacing), labelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.sizeLabels.put(targetName, label);

			// the durations caption labels
			label = new JLabel("D[min]:");
			label.setBounds(optionsLabelStartX, labelStartY + (count * labelYSpacing), smallLabelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.creationDurationLabels.put(targetName, label);

			// the duration text fields
			JTextField textField = new JTextField(
					"" + this.parentMappingManager.getMaxTrainingDurationPreference(targetName), 10);
			textField.setBounds(optionsLabelStartX + smallLabelWidth + smallLabelSpacing,
					labelStartY + (count * labelYSpacing), textFieldWidth, labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.creationDurationTextFields.put(targetName, textField);

			// the size caption labels
			label = new JLabel("TS[#]:");
			label.setBounds(optionsLabelStartX + smallLabelWidth + textFieldWidth + (2 * smallLabelSpacing),
					labelStartY + (count * labelYSpacing), smallLabelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.creationSizeLabels.put(targetName, label);

			// the max training set size text fields
			textField = new JTextField("" + this.parentMappingManager.getReducedTrainingSetPreference(targetName), 10);
			textField.setBounds(optionsLabelStartX + (2 * smallLabelWidth) + textFieldWidth + (3 * smallLabelSpacing),
					labelStartY + (count * labelYSpacing), textFieldWidth, labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.creationSizeTextFields.put(targetName, textField);

			// the FT check boxes
			JCheckBox ftCheckBox = new JCheckBox("FT");
			ftCheckBox.setBounds(
					optionsLabelStartX + (2 * smallLabelWidth) + (2 * textFieldWidth) + (4 * smallLabelSpacing),
					(labelStartY + (count * labelYSpacing)) - ((labelYSpacing - buttonHeight) / 2), textFieldWidth,
					buttonHeight);
			ftCheckBox.setSelected(false);
			ftCheckBox.addActionListener(new FTActionLister(targetName));
			this.creationFTCheckBoxes.put(targetName, ftCheckBox);

			// the RUN buttons
			button = new JButton("Create mapping");
			button.setBounds(buttonStartX,
					(labelStartY + (count * labelYSpacing)) - ((labelYSpacing - buttonHeight) / 2), buttonWidth / 2,
					buttonHeight);
			button.addActionListener(new MappingCreationButtonActionLister(targetName, "RUN"));
			this.creationRunButtons.put(targetName, button);

			// the RETRAIN buttons
			button = new JButton("Retrain mapping");
			button.setBounds(buttonStartX + (buttonWidth / 2),
					(labelStartY + (count * labelYSpacing)) - ((labelYSpacing - buttonHeight) / 2), buttonWidth / 2,
					buttonHeight);
			button.addActionListener(new MappingCreationButtonActionLister(targetName, "RETRAIN"));
			this.creationRetrainButtons.put(targetName, button);

			// the STOP buttons
			button = new JButton("Stop mapping creation");
			button.setBounds(buttonStartX,
					(labelStartY + (count * labelYSpacing)) - ((labelYSpacing - buttonHeight) / 2), buttonWidth,
					buttonHeight);
			button.addActionListener(new MappingCreationButtonActionLister(targetName, "STOP"));
			this.creationStopButtons.put(targetName, button);

			creationPanelHeight = ((labelStartY + (count * labelYSpacing)) - ((labelYSpacing - buttonHeight) / 2))
					+ buttonHeight + 10;

			count++;
		}

		// the panels
		this.creationPanel = new JPanel();
		this.creationPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.creationPanel.setBounds(panelStartX, creationPanelStartY, panelWidth, creationPanelHeight);
		this.creationPanel.setLayout(null);

		this.optimizationPanel = new JPanel();
		this.optimizationPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.optimizationPanel.setBounds(panelStartX, optimizationPanelStartY, panelWidth, optimizationPanelHeight);
		this.optimizationPanel.setLayout(null);

		this.setPreferredSize(new Dimension((2 * panelStartX) + this.optimizationPanel.getWidth(),
				(3 * optimizationPanelStartY) + this.optimizationPanel.getHeight() + this.creationPanel.getHeight()));

	}

	public void refresh() {

		this.add(this.optimizationPanel);
		this.optimizationPanel.removeAll();
		this.optimizationPanel.validate();

		if (this.parentMappingManager.getOptimizationState().equals(TThreadState.IDLE)) {
			this.optimizationPanel.add(this.optimizationButtons.get("RUN"));
			this.optimizationButtons.get("RUN")
					.setEnabled(this.parentMappingManager.areAllMappingHoldersReady()
							&& !this.parentMappingManager.isOptimizationRunning()
							&& this.parentMappingManager.isSensitivityAnalysisIdle());
			this.optimInfoLabel.setText("" + this.parentMappingManager.getMappingMemory().getNumIndividuals()
					+ " mapping-based designs ready for verification!");
			this.optimizationPanel.add(this.optimInfoLabel);
			this.optimizationPanel.add(this.optimUECCheckBox);
			this.optimUECCheckBox.setEnabled(this.optimizationButtons.get("RUN").isEnabled());
		}
		if (this.parentMappingManager.getOptimizationState().equals(TThreadState.RUNNING)) {
			this.optimizationPanel.add(this.optimizationButtons.get("STOP"));
			this.optimProgressBar.setValue(this.parentMappingManager.getOptimizationProgressValue());
			this.optimizationPanel.add(this.optimProgressBar);
		}
		if (this.parentMappingManager.getOptimizationState().equals(TThreadState.READY)) {
			this.optimizationPanel.add(this.optimizationButtons.get("VERIFY"));
			this.optimizationPanel.add(this.optimizationButtons.get("DISCARD"));
			this.optimInfoLabel.setText("" + this.parentMappingManager.getMappingMemory().getNumIndividuals()
					+ " mapping-based designs ready for verification!");
			this.optimizationPanel.add(this.optimInfoLabel);
			int max = Math.min(this.parentMappingManager.getProblem().getNumObjectives() * 100,
					this.parentMappingManager.getMappingMemory().getNumIndividuals());
			int def = Math.min(max / 2, 100);
			this.optimSolutionsToReevaluateSpinner.setModel(new SpinnerNumberModel(def, 1, max, 50));
			this.optimizationPanel.add(this.optimSolutionsToReevaluateSpinner);
			this.optimizationPanel.add(this.optimUECCheckBox);
			this.optimUECCheckBox.setEnabled(false);
		}

		this.optimizationPanel.validate();
		this.optimizationPanel.repaint();

		this.add(this.creationPanel);
		this.creationPanel.removeAll();
		this.creationPanel.validate();
		for (String targetName : this.parentMappingManager.getTargetNames()) {

			this.qualityLabels.get(targetName).setText(this.parentMappingManager.getMappingQuality(targetName));
			this.sizeLabels.get(targetName).setText(this.parentMappingManager.getNrOfTrainingSamples(targetName));
			this.creationPanel.add(this.nameLabels.get(targetName));
			this.creationPanel.add(this.qualityLabels.get(targetName));
			this.creationPanel.add(this.sizeLabels.get(targetName));

			this.creationPanel.add(this.creationDurationLabels.get(targetName));
			this.creationPanel.add(this.creationSizeLabels.get(targetName));

			if (!this.parentMappingManager.isHolderBusy(targetName)) {
				this.creationRunButtons.get(targetName)
						.setEnabled(!this.parentMappingManager.isOptimizationRunning()
								&& this.parentMappingManager.hasSufficientSamples(targetName)
								&& this.parentMappingManager.isSensitivityAnalysisIdle());
				this.creationRetrainButtons.get(targetName)
						.setEnabled(!this.parentMappingManager.isOptimizationRunning()
								&& this.parentMappingManager.isMappingCreated(targetName)
								&& this.parentMappingManager.isSensitivityAnalysisIdle());
				this.creationPanel.add(this.creationRunButtons.get(targetName));
				this.creationPanel.add(this.creationRetrainButtons.get(targetName));
			} else {
				this.creationRunButtons.get(targetName).setEnabled(false);
				this.creationRetrainButtons.get(targetName).setEnabled(false);
				this.creationStopButtons.get(targetName).setEnabled(true);
				this.creationPanel.add(this.creationStopButtons.get(targetName));
			}

			this.creationDurationTextFields.get(targetName)
					.setText("" + this.parentMappingManager.getMaxTrainingDurationPreference(targetName));
			this.creationSizeTextFields.get(targetName)
					.setText("" + this.parentMappingManager.getReducedTrainingSetPreference(targetName));
			this.creationPanel.add(this.creationDurationTextFields.get(targetName));
			this.creationPanel.add(this.creationSizeTextFields.get(targetName));
			this.creationDurationTextFields.get(targetName)
					.setEnabled(this.creationRunButtons.get(targetName).isEnabled()
							|| this.creationRetrainButtons.get(targetName).isEnabled());
			this.creationSizeTextFields.get(targetName).setEnabled(this.creationRunButtons.get(targetName).isEnabled()
					|| this.creationRetrainButtons.get(targetName).isEnabled());

			this.creationPanel.add(this.creationFTCheckBoxes.get(targetName));
			this.creationFTCheckBoxes.get(targetName).setEnabled(this.creationRunButtons.get(targetName).isEnabled());
		}
		for (JLabel hl : this.creationHeaderLabels) {
			this.creationPanel.add(hl);
		}
		for (JSeparator s : this.separators) {
			this.creationPanel.add(s);
		}
		this.creationPanel.validate();
		this.creationPanel.repaint();

		this.validate();
	}
}
