#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 9521 $
$LastChangedDate:: 2019-12-22 14:52:00 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import subprocess
import os
import shutil
import time
from symspace.generic_gui_qt import GUIFormThread
from symspace.osutils.pathutils import symspace_home
from symspace.console.script_utils import ScriptUtils


def openExplorer(file):
    """Open the function directory in the system file manager"""
    try:
        if sys.platform == "win32":
            '''Windows'''
            subprocess.call("explorer " + file)
            # subprocess.call("cmd /c start " + self.fnc_path)
        elif sys.platform == "linux" or sys.platform == "linux2":
            '''linux'''
            raise NotImplementedError
        elif sys.platform == "darwin":
            ''' OS X'''
            raise NotImplementedError
    except Exception as e:
        print(e)
        print('Unable to open dir')



def main(project, parameter):
    if project is None:
        startdir = symspace_home
    else:
        startdir = project.getDirectory() + '\\'
    form_title = 'Create new Python Script'
    form_elements = [('Script name', 'browse_file', startdir, 'Project files (*.ipynb)'),
                     ('Open in editor', 'checkbox'),
                     ('Open folder', 'checkbox'),
                     ]
    form = GUIFormThread(form_title, form_elements)
    form.start()

    if form.accept:
        script = form.result['Script name']
        openeditor = form.result['Open in editor']
        openfolder = form.result['Open folder']

        if not script.endswith('.py'):
            script += '.py'

        template = os.path.join(symspace_home, 'System', 'Python', 'resources', 'NewScript',
                                'new_script.py')
        shutil.copyfile(template, script)

        if openeditor:
            openExplorer(script)
        if openfolder:
            openExplorer(os.path.dirname(script))

    else:
        print('Cancelled by user')


if __name__ == '__main__':
    """
    This script creates a new Python script from a template rile
    """

    ScriptUtils.runWithParams(main, locals(), needs_project=False)
